﻿using System;
using System.ComponentModel.DataAnnotations;

namespace SECOM.Common.Models.Public
{
    public class EncryptionKeyObject
    {

        /// <summary>
        /// The complete EnvelopeKeyObject being uploaded to receiver including data and message properties
        /// </summary>
        [Required]
        public EnvelopeKeyObject Envelope { get; set; }

        /// <summary>
        /// The signature ot the EnvelopeKeyObject in HEX format without whitespace or linebreaks
        /// </summary>
        [Required]
        public string EnvelopeSignature { get; set; }

    }

    public class EnvelopeKeyObject
    {
        /// <summary>
        /// The protected symmetric encryption key
        /// </summary>
        [Required]
        public byte[] EncryptionKey { get; set; }

        /// <summary>
        /// IS THIS NEEDED? Currently not in the standard
        /// </summary>
        [Required]
        public byte[] IV { get; set; }

        /// <summary>
        /// Identifier to the transaction with the encrypted data
        /// </summary>
        [Required]
        public Guid TransactionIdentifier { get; set; }

        /// <summary>
        /// 
        /// </summary>
        [Required]
        public DigitalSignatureValueObject DigitalSignatureValue { get; set; }

        /// <summary>
        /// The public certificate of the sender, used to verify the EnvelopeKeyObject signature
        /// </summary>
        [Required]
        public string EnvelopeSignatureCertificate { get; set; }

        /// <summary>
        /// Claimed Thumbprint for Signed Root Key (X.509 Certificate)
        /// </summary>
        [Required]
        public string EnvelopeRootCertificateThumbprint { get; set; }

        /// <summary>
        /// Time when encryptionKey envelope is signed
        /// </summary>
        [Required]
        public DateTime EnvelopeSignatureTime { get; set;}

    }

    
}