﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Text;

namespace SECOM.Common.Models.Public
{
    public class EncryptionKeyRequestObject
    {
        /// <summary>
        /// The complete EnvelopeKeyRequestObject being uploaded to receiver including data and message properties
        /// </summary>
        [Required]
        public EnvelopeKeyRequestObject Envelope { get; set; }

        /// <summary>
        /// The signature ot the EnvelopeKeyRequestObject in HEX format without whitespace or linebreaks
        /// </summary>
        [Required]
        public string EnvelopeSignature { get; set; }

    }

    public class EnvelopeKeyRequestObject
    {
        /// <summary>
        /// Information identifier retrieved by using reference from response in Get Summary request (UUID)
        /// </summary>
        [Required]
        public Guid DataReference { get; set; }

        /// <summary>
        /// Public certificate of data consumer for deriving shared symmetric key 
        /// </summary>
        [Required]
        public string PublicCertificate { get; set; }

        /// <summary>
        /// The public certificate of the sender, used to verify the EnvelopeKeyObject signature
        /// </summary>
        [Required]
        public string EnvelopeSignatureCertificate { get; set; }

        /// <summary>
        /// Time when encryptionKey envelope is signed
        /// </summary>
        [Required]
        public DateTime EnvelopeSignatureTime { get; set; }
    }
}
