﻿using System;
using System.ComponentModel.DataAnnotations;

namespace SECOM.Common.Models.Public
{
    /// <summary>
    /// The exchange metadata contains information regarding protection scheme, compression, signature and claimed identity.
    /// </summary>
    [Serializable]
    public class ExchangeMetadata
    {
        /// <summary>
        /// (S-100) Indicates if the data is encrypted. 
        /// 0 indicates unencrypted data
        /// 1 indicates encrypted data
        /// </summary>
        [Required]
        public bool DataProtection { get; set; }

        /// <summary>
        /// (S-100) Specification or method used for data protection Such as S-63, SECOM
        /// </summary>
        [Required]
        public string ProtectionScheme { get; set; }

        /// <summary>
        /// (S-100) Specifies the algorithm used to compute digitalSignatureValue Such as “dsa”
        /// </summary>
        [Required]
        public string DigitalSignatureReference { get; set; }

        /// <summary>
        /// (S100) Signed Public Key plus the digital signature
        /// </summary>
        [Required]
        public DigitalSignatureValueObject DigitalSignatureValue { get; set; }


        /// <summary>
        /// (S-100) Indicates if data is compressed.
        /// 0 indicates uncompressed
        /// 1 indicates compressed
        /// </summary>
        [Required]
        public bool CompressionFlag { get; set; }
    }
}
