﻿using System;
using System.ComponentModel.DataAnnotations;

namespace SECOM.Common.Models.Public
{
    public class SummaryObject
    {
        /// <summary>
        /// Reference to data
        /// </summary>
        [Required]
        public string DataReference { get; set; }

        /// <summary>
        /// Flag indicating if data is encrypted or not
        /// </summary>
        [Required]
        public bool DataProtection { get; set; }

        /// <summary>
        /// Flag indicating if data is compressed or not
        /// </summary>
        [Required]
        public bool DataCompression { get; set; }

        /// <summary>
        /// Container type e.g. S100_DataSet, S100_ExchageSet
        ///  </summary>
        public ContainerTypeEnum ContainerType { get; set; }

        /// <summary>
        /// Data product type name requested, e.g. S-122
        /// </summary>
        public Secom_DataProductTypeEnum DataProductType { get; set; }

        /// <summary>
        /// S-100 based Product version e.g. 1.0.0
        /// </summary>
        public string Info_productVersion { get; set; }

        /// <summary>
        /// Identifier of the information object
        /// </summary>
        public string Info_identifier { get; set; }

        /// <summary>
        /// Name of the information object
        /// </summary>
        public string Info_name { get; set; }

        /// <summary>
        /// Status of the information object
        /// </summary>
        public string Info_Status { get; set; }

        /// <summary>
        /// Description of the information object
        /// </summary>
        [Required]
        public string Info_Description { get; set; }

        /// <summary>
        /// Date for last modified
        /// </summary>
        public DateTime? Info_lastModifiedDate { get; set; }

        /// <summary>
        /// Size of the data
        /// </summary>
        public long? Info_size { get; set; }

    }
}