﻿using System;
using System.ComponentModel.DataAnnotations;

namespace SECOM.Common.Models.Public
{
    public class EnvelopeLinkObject
    {
        /// <summary>
        /// Data type of message in the data object
        ///  </summary>
        [Required]
        public ContainerTypeEnum ContainerType { get; set; }

        /// <summary>
        /// Data product type name requested, e.g. S122
        /// </summary>
        [Required]
        public Secom_DataProductTypeEnum DataProductType { get; set; }

        /// <summary>
        /// Service exchange metadata with signature, id etc.
        /// </summary>
        [Required]
        public ExchangeMetadata ExchangeMetadata { get; set; }

        /// <summary>
        /// Flag to indicate whether the data has been uploaded within an active subscription or not.
        /// </summary>
        [Required]
        public bool FromSubscription { get; set; }

        /// <summary>
        /// Indicates request for acknowledgement.
        /// </summary>
        [Required]
        public AckRequestEnum AckRequest { get; set; }


        /// <summary>
        /// Transaction identifier to be used in acknowledgement and retrieving the message
        /// </summary>
        [Required]
        public Guid TransactionIdentifier { get; set; }

        /// <summary>
        /// The public certificate of the sender, used to verify the envelopeLinkObject signature
        /// </summary>
        [Required]
        public string EnvelopeSignatureCertificate { get; set; }

        /// <summary>
        /// Claimed Thumbprint for Signed Root Key (X.509 Certificate)
        /// </summary>
        [Required]
        public string EnvelopeRootCertificateThumbprint { get; set; }

        /// <summary>
        /// Approximated maximum size of the data file in kBytes to be downloaded.
        /// </summary>
        [Required]
        public int Size { get; set; }

        /// <summary>
        /// DateTime when data will be deleted on server. The data need to be fetched before this time.
        /// </summary>
        [Required]
        public DateTime TimeToLive { get; set; }

        /// <summary>
        /// Time stamp when the envelope is signed
        /// </summary>
        [Required]
        public DateTime EnvelopeSignatureTime { get; set; }

    }

    public class UploadLinkObject
    {
        /// <summary>
        /// The complete EnvelopeLinkObject being uploaded to receiver including data and message properties
        /// </summary>
        [Required]
        public EnvelopeLinkObject Envelope { get; set; }

        /// <summary>
        /// The signature ot the EnvelopeLinkObject in HEX format without whitespace or linebreaks
        /// </summary>
        [Required]
        public string EnvelopeSignature { get; set; }
    }
}
