﻿using System;
using System.ComponentModel.DataAnnotations;

namespace SECOM.Common.Models.Public
{
    [Serializable]
    public class EnvelopeUploadObject
    {
        /// <summary>
        /// The payload XML (e.g. S100_ExchangeSet, S100_DataSet), ZIP or binary The data can be open, protected and/or compressed.
        /// </summary>
        [Required]
        public byte[] Data { get; set; }

        /// <summary>
        /// Data type of message in the data object
        ///  </summary>
        [Required]
        public ContainerTypeEnum ContainerType { get; set; }

        /// <summary>
        /// Data product type name requested, e.g. S122
        /// </summary>
        [Required]
        public Secom_DataProductTypeEnum DataProductType { get; set; }

        /// <summary>
        /// The exchange metadata contains information regarding protection scheme, compression, signature and claimed identity.
        /// </summary>
        [Required]
        public ExchangeMetadata ExchangeMetadata { get; set; }

        /// <summary>
        /// Flag to indicate whether the data has been uploaded within an active subscription or not.
        /// </summary>
        [Required]
        public bool FromSubscription { get; set; }

        /// <summary>
        /// Indicates request for acknowledgement.
        /// </summary>
        [Required]
        public AckRequestEnum AckRequest { get; set; }

        /// <summary>
        /// Transaction identifier to be used in acknowledgement
        /// </summary>
        [Required]
        public Guid TransactionIdentifier { get; set; }

        /// <summary>
        /// The public certificate of the sender. used to verify the envelopeObject signature
        /// </summary>
        [Required]
        public string EnvelopeSignatureCertificate { get; set; }

        /// <summary>
        /// Claimed Thumbprint for Signed Root Key (X.509 Certificate)
        /// </summary>
        [Required]
        public string EnvelopeRootCertificateThumbprint { get; set; }

        /// <summary>
        /// Time stamp when the envelope is signed
        /// </summary>
        [Required]
        public DateTime EnvelopeSignatureTime { get; set; }

    }

    [Serializable]
    public class UploadObject
    {
        /// <summary>
        /// The complete EnvelopeObject being uploaded to receiver including data and message properties
        /// </summary>
        [Required]
        public EnvelopeUploadObject Envelope { get; set; }

        /// <summary>
        /// The signature ot the EnvelopeObject in HEX format without whitespace or linebreaks
        /// </summary>
        [Required]
        public string EnvelopeSignature { get; set; }


    }
}