﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Text;
using SharpCompress.Archives;
using SharpCompress.Archives.SevenZip;
using SharpCompress.Archives.Tar;
using SharpCompress.Common;
using SharpCompress.Writers;

namespace SECOM.Common.Services
{
    public interface ICompressionService
    {
        byte[] Compress(byte[] data);
        byte[] Decompress(byte[] data);
        byte[] Archive(string path);
        void Dearchive(string path, byte[] tarData);
    }

    public class CompressionService : ICompressionService
    {
        public byte[] Compress(byte[] data)
        {
            using (var inputStream = new MemoryStream(data))
            using (var outputStream = new MemoryStream())
            {
                using (var gzipStream = new GZipStream(outputStream, CompressionMode.Compress))
                {
                    inputStream.CopyTo(gzipStream);
                }

                return outputStream.ToArray();
            }
        }

        public byte[] Decompress(byte[] data)
        {
            using (var inputStream = new MemoryStream(data))
            using (var outputStream = new MemoryStream())
            {
                try
                {
                    using (var gzipStream = new GZipStream(inputStream, CompressionMode.Decompress))
                    {
                        gzipStream.CopyTo(outputStream);
                    }

                    return outputStream.ToArray();
                }
                catch (Exception)
                { 
                    // Do nothing 
                    return Encoding.UTF8.GetBytes("Error when decompressing message");
                }
            }
        }

        public byte[] Archive(string path)
        {
            using (var outputStream = new MemoryStream())
            using (var archive = TarArchive.Create())
            {
                archive.AddAllFromDirectory(path);
                archive.SaveTo(outputStream, new WriterOptions(CompressionType.None));
                return outputStream.ToArray();
            }
        }

        public void Dearchive(string path, byte[] tarData)
        {
            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            using (var inputStream = new MemoryStream(tarData))
            using (var archive = TarArchive.Open(inputStream))
            foreach (var entry in archive.Entries.Where(entry => !entry.IsDirectory))
            {
                entry.WriteToDirectory(path, new ExtractionOptions()
                {
                    ExtractFullPath = true,
                    Overwrite = true
                });
            }
        }
    }
}