﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using SECOM.Common.Integration.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Security.Cryptography.X509Certificates;
using System.Text.Json;
using System.Threading.Tasks;

namespace SECOM.Common.Services
{
    public interface IIdentityRegistryService
    {
        Task<IdRegistryResponeObject> FindIdentities();
        bool IsCertificateValid(X509Certificate2 cert);
    }

    public class IdentityRegistryService : IIdentityRegistryService
    {
        private readonly ILogger<IdentityRegistryService> _logger;
        public readonly HttpClient _client;

        public IdentityRegistryService(ILogger<IdentityRegistryService> logger,
            IHttpClientFactory clientFactory,
            IHttpContextAccessor contextAccessor)
        {
            _logger = logger;
            _client = clientFactory.CreateClient("IdentityRegistry");

            _logger.LogDebug($"{GetType().FullName} accessed by user {contextAccessor.HttpContext.User.Identity.Name} from {contextAccessor.HttpContext.Connection.RemoteIpAddress}");
        }

        public async Task<IdRegistryResponeObject> FindIdentities()
        {
            _logger.LogInformation($"Sending findIdentities request to IdentityRegistry");

            try
            {
                var url = "/x509/api/orgs?page=0&size=1000";
                var response = await _client.GetAsync(url);
                response.EnsureSuccessStatusCode();

                using var responseStream = await response.Content.ReadAsStreamAsync();
                {
                    var serializeOptions = new JsonSerializerOptions
                    {
                        PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
                    };

                    return await JsonSerializer.DeserializeAsync<IdRegistryResponeObject>
                        (responseStream, serializeOptions);
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex.Message, ex);
                throw;
            }
        }

        public bool IsCertificateValid(X509Certificate2 cert)
        {
            throw new NotImplementedException();
        }
    }
}
