﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using SECOM.Common.Integration.Models;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace SECOM.Common.Services
{
    public interface IServiceRegistryService
    {
        Task<List<FindServiceResponseObject>> FindService(string query, string geometry, int page, int pageSize);
    }

    public class ServiceRegistryService : IServiceRegistryService
    {
        private readonly ILogger<ServiceRegistryService> _logger;
        public readonly HttpClient _client;

        public ServiceRegistryService(ILogger<ServiceRegistryService> logger,
            IHttpClientFactory clientFactory,
            IHttpContextAccessor contextAccessor)
        {
            _logger = logger;
            _client = clientFactory.CreateClient("ServiceRegistry");

            _logger.LogDebug($"{GetType().FullName} accessed by user {contextAccessor.HttpContext.User.Identity.Name} from {contextAccessor.HttpContext.Connection.RemoteIpAddress}");
        }

        public async Task<List<FindServiceResponseObject>> FindService(string query, string geometry, int page, int pageSize)
        {
            _logger.LogInformation($"Sending findServices request to ServiceRegistry");

            try
            {
                var url = $"/api/_search/serviceInstance?query={query}";

                var response = await _client.GetAsync(url);
                response.EnsureSuccessStatusCode();

                using var responseStream = await response.Content.ReadAsStreamAsync();
                {
                    var serializeOptions = new JsonSerializerOptions
                    {
                        PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
                    };

                    return await JsonSerializer.DeserializeAsync<List<FindServiceResponseObject>>
                        (responseStream, serializeOptions);
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex.Message, ex);
                throw;
            }
        }
    }
}