﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Reflection;
using System.IO;

namespace SECOM.Common.Utils
{
    public class CustomSerializer
    {
        private Type _type;
        private string _result;

        public CustomSerializer(Type type)
        {
            _type = type;
            _result = "";
        }

        /// <summary>
        /// Nested serializer for objects of max 2 level depth
        /// </summary>
        /// <param name="graph">Object to be serialized</param>
        /// <returns></returns>
        public string Serialize(object graph)
        {
            try
            {
                List<PropertyInfo> root = _type.GetProperties().ToList();

                foreach (var rootItem in root)
                {
                    if (!IsLeaf(rootItem, graph))
                    {
                        object level1Value = rootItem.GetValue(graph, null);
                        List<PropertyInfo> level1 = level1Value.GetType().GetProperties()?.ToList();
                        foreach (var level1Item in level1)
                        {
                            if (!IsLeaf(level1Item, level1Value))
                            {
                                object level2Value = level1Item.GetValue(level1Value, null);
                                List<PropertyInfo> level2 = level2Value.GetType().GetProperties()?.ToList();
                                foreach (var level2Item in level2)
                                {
                                    if (!IsLeaf(level2Item, level2Value)) { }
                                }
                            }
                        }
                    }
                }

                return _result.Substring(0, _result.Length - 1);
            }
            catch (Exception ex) { throw; }
        }

        private bool IsLeaf(PropertyInfo propertyInfo, object graph)
        {
            if (propertyInfo.GetValue(graph, null) != null && !string.IsNullOrEmpty(propertyInfo.GetValue(graph, null).ToString()))
            {
                if (propertyInfo.PropertyType.IsPrimitive || propertyInfo.PropertyType == typeof(string))
                {
                    if (propertyInfo.PropertyType == typeof(bool))
                    {
                        _result += $"{propertyInfo.GetValue(graph).ToString().ToLower()}.";
                    }
                    else
                    {
                        _result += $"{propertyInfo.GetValue(graph)}.";
                    }
                    return true;

                }
                if (propertyInfo.PropertyType == typeof(DateTime))
                {
                    var dt = new DateTimeOffset((DateTime)propertyInfo.GetValue(graph, null));
                    _result += $"{dt.ToUnixTimeSeconds()}.";
                    return true;
                }
                if (propertyInfo.PropertyType == typeof(byte[]))
                {
                    var bytes = propertyInfo.GetValue(graph, null) as byte[];

                    _result += $"{Convert.ToBase64String(bytes)}.";
                    return true;
                }
                if (propertyInfo.PropertyType == typeof(Guid))
                {
                    _result += $"{propertyInfo.GetValue(graph)}.";
                    return true;
                }
                if (propertyInfo.PropertyType == typeof(Models.Public.AckRequestEnum))
                {
                    _result += $"{(int)propertyInfo.GetValue(graph)}.";
                    return true;
                }
                if (propertyInfo.PropertyType == typeof(Models.Public.AckTypeEnum))
                {
                    _result += $"{(int)propertyInfo.GetValue(graph)}.";
                    return true;
                }
                if (propertyInfo.PropertyType == typeof(Models.Public.Secom_DataProductTypeEnum))
                {
                    _result += $"{(int)propertyInfo.GetValue(graph)}.";
                    return true;
                }
                if (propertyInfo.PropertyType == typeof(Models.Public.ContainerTypeEnum))
                {
                    _result += $"{(int)propertyInfo.GetValue(graph)}.";
                    return true;
                }
                if (propertyInfo.PropertyType.IsGenericType && 
                    propertyInfo.PropertyType.GetGenericTypeDefinition() == typeof(Nullable<>))
                {
                    if(typeof(Models.Public.NackTypeEnum) == Nullable.GetUnderlyingType(propertyInfo.PropertyType))
                    _result += $"{(int)propertyInfo.GetValue(graph)}.";
                    return true;
                }
            }
            else //rule for null value
            {
                _result += $".";
                return true;
            }

            return false;
        }
    }
}
