﻿using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Design;
using SECOM.DataAccess.Entities;
using System;
using System.Collections.Generic;
using System.IO;

namespace SECOM.DataAccess
{
    public class SecomDbContext : DbContext
    {
        public SecomDbContext(DbContextOptions options) : base(options)
        {
        }

        public DbSet<UploadedMessage> UploadedMessage { get; set; }

        public DbSet<SentMessage> SentMessage { get; set; }
        public DbSet<PublishedMessage> PublishedMessage { get; set; }
        public DbSet<EncryptionKey> EncryptionKey { get; set; }
        public DbSet<Identity> Identity { get; set; }
        public DbSet<Notification> Notification { get; set; }
        public DbSet<Acknowledgement> Acknowledgement { get; set; }
        public DbSet<AckEnum> AckEnum { get; set; }
        public DbSet<AckRequestEnum> AckRequestEnum { get; set; }
        public DbSet<ContainerTypeEnum> ContainerTypeEnum { get; set; }
        public DbSet<Secom_DataProductTypeEnum> DataProductTypeEnum { get; set; }
        public DbSet<TemporaryKey> TemporaryKey { get; set; }
        public DbSet<IdentityAcl> IdentityAcl { get; set; }
        public DbSet<Capability> Capability { get; set; }

        protected override void OnModelCreating(ModelBuilder builder)
        {
            builder.Seed(new List<AckEnum>{
                new Entities.AckEnum {Id = 0, Value = "Delivered" },
                new Entities.AckEnum { Id = 1, Value = "Opened" },
                new Entities.AckEnum { Id = 2, Value = "Error" }
            });

            builder.Seed(new List<NackEnum>
            {
                new Entities.NackEnum{Id = 0, Value = "XML Schema validation error"},
                new Entities.NackEnum{Id = 1, Value = "Unknown data type or version"},
                new Entities.NackEnum{Id = 2, Value = "Failed signature verification"},
                new Entities.NackEnum{Id = 3, Value = "Failed decryption"},
                new Entities.NackEnum{Id = 4, Value = "Failed decompression"}
            });

            builder.Seed(new List<AckRequestEnum> {
                new Entities.AckRequestEnum { Id = 0, Value= "No ACK requested" },
                new Entities.AckRequestEnum { Id = 1, Value = "Delivered ACK requested" },
                new Entities.AckRequestEnum { Id = 2, Value = "Opened ACK requested" },
                new Entities.AckRequestEnum { Id = 3, Value = "Delivered + Opened ACK requested" }
            });

            builder.Seed(new List<ContainerTypeEnum>
            {
                new Entities.ContainerTypeEnum { Id = 0, Value = "S100_DATASET" },
                new Entities.ContainerTypeEnum { Id = 1, Value = "S100_EXCHANGESET" },
                new Entities.ContainerTypeEnum { Id = 2, Value = "NONE" },
            });

            builder.Seed(new List<Secom_DataProductTypeEnum>
            {
                new Entities.Secom_DataProductTypeEnum { Id = 0, Value = "S57" },
                new Entities.Secom_DataProductTypeEnum { Id = 1, Value = "S101" },
                new Entities.Secom_DataProductTypeEnum { Id = 2, Value = "S102" },
                new Entities.Secom_DataProductTypeEnum { Id = 3, Value = "S104" },
                new Entities.Secom_DataProductTypeEnum { Id = 4, Value = "S111" },
                new Entities.Secom_DataProductTypeEnum { Id = 5, Value = "S122" },
                new Entities.Secom_DataProductTypeEnum { Id = 6, Value = "S123" },
                new Entities.Secom_DataProductTypeEnum { Id = 7, Value = "S124" },
                new Entities.Secom_DataProductTypeEnum { Id = 8, Value = "S125" },
                new Entities.Secom_DataProductTypeEnum { Id = 9, Value = "S126" },
                new Entities.Secom_DataProductTypeEnum { Id = 10, Value = "S127" },
                new Entities.Secom_DataProductTypeEnum { Id = 11, Value = "S128" },
                new Entities.Secom_DataProductTypeEnum { Id = 12, Value = "S129" },
                new Entities.Secom_DataProductTypeEnum { Id = 13, Value = "S210" },
                new Entities.Secom_DataProductTypeEnum { Id = 14, Value = "S211" },
                new Entities.Secom_DataProductTypeEnum { Id = 15, Value = "S212" },
                new Entities.Secom_DataProductTypeEnum { Id = 16, Value = "S401" },
                new Entities.Secom_DataProductTypeEnum { Id = 17, Value = "S402" },
                new Entities.Secom_DataProductTypeEnum { Id = 18, Value = "S411" },
                new Entities.Secom_DataProductTypeEnum { Id = 19, Value = "S412" },
                new Entities.Secom_DataProductTypeEnum { Id = 20, Value = "S413" },
                new Entities.Secom_DataProductTypeEnum { Id = 21, Value = "S414" },
                new Entities.Secom_DataProductTypeEnum { Id = 22, Value = "S421" },
                new Entities.Secom_DataProductTypeEnum { Id = 23, Value = "RTZ" },
                new Entities.Secom_DataProductTypeEnum { Id = 24, Value = "EPC" },
            });


            builder.Seed(new List<SECOM_ResponseCodeEnum>
            {
                new SECOM_ResponseCodeEnum { Id = 0, Value = "Missing required data for the service" },
                new SECOM_ResponseCodeEnum { Id = 1, Value = "Failed signature verification" },
                new SECOM_ResponseCodeEnum { Id = 2, Value = "Invalid certificate"},
                new SECOM_ResponseCodeEnum { Id = 3, Value = "Schema validation error" }
            });

            builder.Seed(new List<Capability>
            {
                new Capability
                {
                    Id = 1,
                    ImplementsAccess = true,
                    ImplementsGet = true,
                    ImplementsGetSummary = true,
                    ImplementsSubscription = true,
                    ImplementsUpload = true,
                    ImplementsUploadLink = true,
                    ImplementsGetByLink = true,
                    ImplementsEncryptionKey = true,
                    DataProductType = 24,
                    ProductSchemaUrl = @"http://cirm.org/rtz/RTZ%20Schema%20version%201_0.xsd",
                    ServiceVersion = "1.0"
                },
                new Capability
                {
                    Id = 2,
                    ImplementsAccess = true,
                    ImplementsGet = true,
                    ImplementsGetSummary = true,
                    ImplementsSubscription = true,
                    ImplementsUpload = true,
                    ImplementsUploadLink = true,
                    ImplementsGetByLink = true,
                    ImplementsEncryptionKey = true,
                    DataProductType = 24,
                    ProductSchemaUrl = "http://cirm.org/rtz/RTZ%20Schema%20version%201_1.xsd",
                                        ServiceVersion = "1.0"
                },
                new Capability
                {
                    Id = 3,
                    ImplementsAccess = true,
                    ImplementsGet = true,
                    ImplementsGetSummary = true,
                    ImplementsSubscription = true,
                    ImplementsUpload = true,
                    ImplementsUploadLink = true,
                    ImplementsGetByLink = true,
                    ImplementsEncryptionKey = true,
                    DataProductType = 24,
                    ProductSchemaUrl = "http://cirm.org/rtz/RTZ%20Schema%20version%201_2.xsd",
                                        ServiceVersion = "1.0"
                },
                new Capability
                {
                    Id = 4,
                    ImplementsAccess = true,
                    ImplementsGet = true,
                    ImplementsGetSummary = true,
                    ImplementsSubscription = true,
                    ImplementsUpload = true,
                    ImplementsUploadLink = true,
                    ImplementsGetByLink = true,
                    ImplementsEncryptionKey = true,
                    DataProductType = 23,
                    ProductSchemaUrl = "http://iho.com/S-421/1/0/0",
                                        ServiceVersion = "1.0"
                },
                new Capability
                {
                    Id = 5,
                    ImplementsAccess = false,
                    ImplementsGet = false,
                    ImplementsGetSummary = false,
                    ImplementsSubscription = false,
                    ImplementsUpload = true,
                    ImplementsUploadLink = true,
                    ImplementsGetByLink = false,
                    ImplementsEncryptionKey = true,
                    DataProductType = 8,
                    ProductSchemaUrl = "http://iho.com/S-124/1/0/0",
                                        ServiceVersion = "1.0"
                },
                //new Capability
                //{
                //    Id = 6,
                //    ImplementsGet = false,
                //    ImplementsGetSummary = false,
                //    ImplementsSubscription = false,
                //    ImplementsUpload = true,
                //    ImplementsUploadLink = true,
                //    ImplementsGetByLink = false,
                //    ImplementsEncryptionKey = true,
                //    DataProductTypeName = "S100_EXCHANGESET",
                //    ProductSchemaUrl = "http://iho.com/S-100/1/0/0",
                //                        ServiceVersion = "1.0"
                //}
            });

            //var pathToData = @"\\data\s421.xml";
            //var pub1 = new PublishedMessage
            //{
            //    CreatedTime = DateTime.UtcNow,
            //    DataReference = Guid.NewGuid(),
            //    DataType = 0,
            //    ProductSpecificationId = "S-421",
            //    ProductSpecificationVersion = "1.0.0",
            //    Data = File.ReadAllBytes(pathToData),
            //    ExchangeMetadata = new ExchangeMetadata
            //    {
            //        CompressionFlag = false,
            //        DataProtection = false,
            //        ProtectionScheme = "SECOM",
            //        DigitalSignature = 

            //    }
            //}

            //string data = System.IO.File.ReadAllText(@"..\SECOM.Common\Data\data.rtz");

            //builder.Seed(new List<Identity>
            //{
            //    new Identity { Id = 0, Name = "SMA", Uid = "urn:mrn:mcp:org:navelink-dev:sma" },
            //    new Identity { Id = 1, Name = "SAAB", Uid = "urn:mrn:mcp:org:navelink-dev:saab" },
            //    new Identity { Id = 2, Name = "StormGeo", Uid = "urn:mrn:mcp:org:navelink-dev:stormgeo" }
            //});
        }
    }

    public class SecomDbContextDesignFactory : IDesignTimeDbContextFactory<SecomDbContext>
    {
        public SecomDbContext CreateDbContext(string[] args)
        {
            var optionsBuilder = new DbContextOptionsBuilder<SecomDbContext>()
                .UseNpgsql("User ID=postgres;Password=1234;Host=localhost;Port=5432;Database=SECOM;Pooling=true;");

            return new SecomDbContext(optionsBuilder.Options);
        }
    }
}
