﻿using Org.BouncyCastle.Utilities.Encoders;
using SECOM.Common.Models.Private;
using SECOM.Common.Services;
using SECOM.Module.Ui.Services;
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;

namespace SECOM.Module.Ui.Extensions
{
    public class ExtendedUploadedObject : UploadedMessageDto
    {
        public static ISecomService SecomService;
        public static IEncryptionService EncryptionService;
        public static ICompressionService CompressionService;
        public static ISignatureService SignatureService;
        public static SettingsService SettingsService;

        public ExtendedUploadedObject(UploadedMessageDto obj)
        {
            ExchangeMetadata = obj.ExchangeMetadata;
            AckEndpoint = obj.AckEndpoint;
            AckRequest = obj.AckRequest;
            TechnicalAckDelivered = obj.TechnicalAckDelivered;
            OperationalAckDelivered = obj.OperationalAckDelivered;
            ReceivedTime = obj.ReceivedTime;
            Data = obj.Data;
            ContainerType = obj.ContainerType;
            FromSubscription = obj.FromSubscription;
            TransactionIdentifier = obj.TransactionIdentifier;
            IsExchangeSet = obj.IsExchangeSet;
            
            var task = new TaskFactory().StartNew(() => GetEncryptionKey());
            task.Wait();
            EncryptionKey = task.Result.Result;
            Decrypt();
            Decompress();
            //Dearchive();

            // If exchange set save zip file
            if(IsExchangeSet)
            {
                File.WriteAllBytes(@$"c:\SECOM_Files\{TransactionIdentifier:n}.zip", Data);
            }
        }

        public async Task<EncryptionKeyDto> GetEncryptionKey()
        {
            if (!ExchangeMetadata.DataProtection)
                return null;

            var key = await SecomService.GetEncryptionKey(TransactionIdentifier);
            return key;
        }

        public void Decrypt ()
        {
            if (ExchangeMetadata.DataProtection && !IsDecrypted)
            {
                if (EncryptionKey == null)
                {
                    return;
                }

                var senderPublicKey = new X509Certificate2(Convert.FromBase64String(ExchangeMetadata.DigitalSignature.PublicCertificate));
                var receiverKey = new X509Certificate2(SettingsService.CertificateFile, SettingsService.CertificatePassword);
                var decryptedKey = EncryptionService.DecryptWithDerivedKey(EncryptionKey.Key, receiverKey, senderPublicKey, EncryptionKey.IV);

                var signature = new Common.Models.Public.DigitalSignatureValueObject
                {
                    DigitalSignature = EncryptionKey.DigitalSignature.DigitalSignatureValue,
                    PublicCertificate = EncryptionKey.DigitalSignature.PublicCertificate,
                    PublicRootCertificateThumbprint = EncryptionKey.DigitalSignature.PublicRootCertificateThumbprint
                };

                var json = JsonSerializer.Serialize(signature);
                //var isValid = SignatureService.Verify(signature, decryptedKey, out string result);
                //TODO: Handle invalid signature. What should happen here? 
                //Send NACK?
                //Delete the received encryptionKey object?
                //Delete the corresponding encrypted uploaded message?

                Data = EncryptionService.Decrypt(Data, decryptedKey, EncryptionKey.IV);
                IsDecrypted = true;
            }
        }

        public void Dearchive()
        {
            if (IsExchangeSet)
            {
                CompressionService.Dearchive(@$"c:\SECOM_Files\{TransactionIdentifier:n}", Data);
                Data = Encoding.UTF8.GetBytes(@$"Data is an Exchange folder look in folder c:\SECOM_Files\{ TransactionIdentifier:n}");
            }
        }

        public void Decompress()
        {
            if (ExchangeMetadata.CompressionFlag == true && !IsDecompressed)
            {
                Data = CompressionService.Decompress(Data);
            }
        }

        public string DataAsString
        {
            get
            {
                if (Data == null)
                    return string.Empty;
                if (IsExchangeSet)
                {
                    return @$"Data is an Exchange set look in c:\SECOM_Files\{ TransactionIdentifier:n}.zip";
                }
                return Encoding.UTF8.GetString(Data);
            }
        }

        public bool IsDecrypted { get; set; }
        public bool IsDecompressed { get; set; }
        public EncryptionKeyDto EncryptionKey { get; set; }

        public string KeyAsHex 
        { 
            get
            {
                if (EncryptionKey == null)
                    return string.Empty;

                return Hex.ToHexString(EncryptionKey.Key);
            }
        }
        public string IvAsHex
        {
            get
            {
                if (EncryptionKey == null)
                    return string.Empty;

                return Hex.ToHexString(EncryptionKey.IV);
            }
        }
    }
}
