﻿using AutoMapper;
using SECOM.Common.Integration.Models;
using SECOM.Common.Models.Private;
using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Text.Encodings.Web;
using System.Text.Json;
using System.Threading.Tasks;

namespace SECOM.Module.Ui.Services
{

    public interface ISentMessageService
    {
        Task<bool> Save(SECOM.Common.Models.Public.UploadObject uploadObject, string receiverMrn);
        Task<bool> Save(SECOM.Common.Models.Public.UploadLinkObject uploadObject, byte[] data, string receiverMrn, DateTime timeToLive);
    }

    public class SentMessageService : ISentMessageService
    {
        private readonly HttpClient _client;
        private readonly JsonSerializerOptions _serializeOptions;
        private readonly SettingsService _settingsService;

        public SentMessageService(IHttpClientFactory clientFactory, SettingsService settingsService)
        {
            _settingsService = settingsService;
            _client = clientFactory.CreateClient("SecomPrivateService");
            _serializeOptions = new JsonSerializerOptions
            {
                PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
            };
        }
        public async Task<bool> Save(Common.Models.Public.UploadObject uploadObject, string receiverMrn)
        {
            var stringContent = JsonSerializer.Serialize(uploadObject, _serializeOptions);
            var content = new StringContent(stringContent, Encoding.UTF8, "application/json");
            using (var response = await _client.PostAsync($"{_settingsService.PrivateBaseUrl}/v1/SentMessage?receiverMrn={receiverMrn}", content))
            {
                response.EnsureSuccessStatusCode();
                string jsonString = await response.Content.ReadAsStringAsync();
                var result = JsonSerializer.Deserialize<bool>(jsonString, _serializeOptions);

                return result;
            }
        }

        public async Task<bool> Save(Common.Models.Public.UploadLinkObject uploadObject, byte[] data, string receiverMrn, DateTime timeToLive)
        {
            var o = new Common.Models.Public.UploadObject
            {
                Envelope = new Common.Models.Public.EnvelopeUploadObject
                {
                    //AckEndpoint = null,
                    AckRequest = Common.Models.Public.AckRequestEnum.No_ACK_requested,
                    Data = data,
                    ContainerType = uploadObject.Envelope.ContainerType,
                    EnvelopeSignatureCertificate = uploadObject.Envelope.EnvelopeSignatureCertificate,
                    ExchangeMetadata = uploadObject.Envelope.ExchangeMetadata,
                    FromSubscription = false,
                    TransactionIdentifier = uploadObject.Envelope.TransactionIdentifier
                },
                EnvelopeSignature = uploadObject.EnvelopeSignature
            };

            var stringContent = JsonSerializer.Serialize(o, _serializeOptions);
            var content = new StringContent(stringContent, Encoding.UTF8, "application/json");

            using (var response = await _client.PostAsync($"{_settingsService.PrivateBaseUrl}/v1/SentMessage/link?receiverMrn={receiverMrn}&timeToLive={timeToLive}", content))
            {
                response.EnsureSuccessStatusCode();
                string jsonString = await response.Content.ReadAsStringAsync();
                var result = JsonSerializer.Deserialize<bool>(jsonString, _serializeOptions);

                return result;
            }
        }
    }
}