﻿using System;
using System.ComponentModel;
using System.Linq.Expressions;

namespace SECOM.Module.Ui.Services
{
    public class SettingsService : INotifyPropertyChanged
    {
        public SettingsService()
        {

        }

        private string _instanceName = "";
        public string InstanceName
        {
            get
            {
                return _instanceName;
            }
            set
            {
                if (_instanceName == value)
                    return;

                _instanceName = value;

                OnPropertyChanged(() => InstanceName);
                OnSettingsChanged();
            }
        }

        private string _instanceDisplayName = "Not selected";
        public string InstanceDisplayName
        {
            get
            {
                return _instanceDisplayName;
            }
            set
            {
                if (_instanceDisplayName == value)
                    return;

                _instanceDisplayName = value;

                OnPropertyChanged(() => InstanceDisplayName);
            }
        }

        private string _environment = "Not selected";
        public string Environment
        {
            get
            {
                return _environment;
            }
            set
            {
                if (_environment == value)
                    return;

                _environment = value;

                OnSettingsChanged();
                OnPropertyChanged(() => Environment);
            }
        }

        private string _privateBaseUrl;
        public string PrivateBaseUrl
        {
            get
            {
                return _privateBaseUrl;
            }
            set
            {
                if (_privateBaseUrl == value)
                    return;

                _privateBaseUrl = value;

                OnSettingsChanged();
                OnPropertyChanged(() => PrivateBaseUrl);
            }
        }

        private string _publicBaseUrl;
        public string PublicBaseUrl
        {
            get
            {
                return _publicBaseUrl;
            }
            set
            {
                if (_publicBaseUrl == value)
                    return;
                _publicBaseUrl = value;

                OnSettingsChanged();
                OnPropertyChanged(() => PublicBaseUrl);
            }
        }

        private string _selectedServiceUrl;
        public string SelectedServiceUrl
        {
            get
            {
                return _selectedServiceUrl;
            }
            set
            {
                if (_selectedServiceUrl == value)
                    return;

                _selectedServiceUrl = value;
                OnPropertyChanged(() => SelectedServiceUrl);
                OnSelectedUrlChangedChanged();
            }
        }

        private string _selectedServiceOrg;
        public string SelectedServiceOrg
        {
            get
            {
                return _selectedServiceOrg;
            }
            set
            {
                if (_selectedServiceOrg == value)
                    return;

                _selectedServiceOrg = value;
                OnPropertyChanged(() => SelectedServiceOrg);
                OnSelectedUrlChangedChanged();
            }
        }

        private string _certificateFile;
        public string CertificateFile
        {
            get
            {
                return _certificateFile;
            }
            set
            {
                if (_certificateFile == value)
                    return;

                _certificateFile = value;

                OnSettingsChanged();
                OnPropertyChanged(() => CertificateFile);
            }
        }

        private string _certificatePassword;
        public string CertificatePassword
        {
            get
            {
                return _certificatePassword;
            }
            set
            {
                if (_certificatePassword == value)
                    return;

                _certificatePassword = value;
                OnSettingsChanged();
                OnPropertyChanged(() => CertificatePassword);
            }
        }

        private string _envelopeCertificateFile;
        public string EnvelopeCertificateFile
        {
            get
            {
                return _envelopeCertificateFile;
            }
            set
            {
                if (_envelopeCertificateFile == value)
                    return;

                _envelopeCertificateFile = value;

                OnSettingsChanged();
                OnPropertyChanged(() => EnvelopeCertificateFile);
            }
        }

        private string _envelopeCertificatePassword;
        public string EnvelopeCertificatePassword
        {
            get
            {
                return _envelopeCertificatePassword;
            }
            set
            {
                if (_envelopeCertificatePassword == value)
                    return;

                _envelopeCertificatePassword = value;
                OnSettingsChanged();
                OnPropertyChanged(() => EnvelopeCertificatePassword);
            }
        }

        public event EventHandler NewMessages;
        public void OnNewMessages()
        {
            NewMessages?.Invoke(this, new EventArgs());
        }

        public event EventHandler SelectedUrlChangedChanged;
        void OnSelectedUrlChangedChanged()
        {
            SelectedUrlChangedChanged?.Invoke(this, new EventArgs());
        }

        public event EventHandler SettingsChanged;

        void OnSettingsChanged()
        {
            SettingsChanged?.Invoke(this, new EventArgs());
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged<TProperty>(Expression<Func<TProperty>> projection)
        {
            var memberExpression = (MemberExpression)projection.Body;
            OnPropertyChangedExplicit(memberExpression.Member.Name);
        }

        void OnPropertyChangedExplicit(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                var e = new PropertyChangedEventArgs(propertyName);
                handler(this, e);
            }
        }
    }

}
