﻿using Microsoft.Extensions.Configuration;
using Org.BouncyCastle.Asn1.Ocsp;
using Org.BouncyCastle.Asn1.X509;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.OpenSsl;
using Org.BouncyCastle.Security;
using Org.BouncyCastle.Utilities.Encoders;
using SECOM.Common.Integration.Models;
using SECOM.Common.Models.Private;
using SECOM.Common.Models.Public;
using SECOM.Common.Services;
using SECOM.Common.Utils;
using SECOM.Module.Ui.Services;
using SECOM.Module.Ui.Utils;
using SECOM.Module.Ui.Views.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Runtime.Serialization.Formatters.Binary;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Text.Json;
using System.Windows;
using System.Windows.Input;
using System.Xml;
using System.Xml.Linq;

namespace SECOM.Module.Ui.ViewModels
{
    public class FindServicesViewModel : ViewModelBase
    {
        #region privateMembers
        private readonly SettingsService _settingsService;
        private readonly ISecomService _secomService;
        private readonly IServiceProvider _serviceProvider;
        private readonly ISignatureService _signatureService;
        private readonly IEncryptionService _encryptionService;
        private readonly ICompressionService _compressionService;
        private readonly ISentMessageService _sentMessageService;
        private readonly ICertificateService _certificateService;
        #endregion

        #region constructor
        public FindServicesViewModel(SettingsService settingsService,
            ISecomService secomService,
            IConfiguration configuration,
            ISignatureService signatureService,
            IEncryptionService encryptionService,
            IServiceProvider serviceProvider,
            ICompressionService compressionService,
            ISentMessageService sentMessageService,
            ICertificateService certificateService)
        {
            _settingsService = settingsService;
            _secomService = secomService;
            _serviceProvider = serviceProvider;
            _signatureService = signatureService;
            _encryptionService = encryptionService;
            _compressionService = compressionService;
            _sentMessageService = sentMessageService;
            _certificateService = certificateService;

            Services = new ObservableCollection<FindServiceResponseObject>();

            var services = configuration.GetValue<string>("FindServiceItems");
            if (services != null)
            {
                try
                {
                    foreach (var service in services.Split(';'))
                    {
                        var tmp = service.Split('#');
                        if (tmp != null && tmp.Length == 2)
                            Services.Add(new FindServiceResponseObject
                            {
                                Name = tmp[0],
                                EndpointUri = tmp[1],
                                InstanceAsXml = new Xml()
                            });
                    }
                }
                catch (Exception)
                { }
            }
        }

        #endregion

        #region properties
        private string _area;
        public string Area
        {
            get
            {
                return _area;
            }
            set
            {
                if (_area == value)
                    return;
                _area = value;

                OnPropertyChanged(() => Area);
            }
        }

        private string _unloCode;
        public string UnloCode
        {
            get
            {
                return _unloCode;
            }
            set
            {
                if (_unloCode == value)
                    return;
                _unloCode = value;

                OnPropertyChanged(() => UnloCode);
            }
        }

        private string _serviceProviderIds;
        public string ServiceProviderIds
        {
            get
            {
                return _serviceProviderIds;
            }
            set
            {
                if (_serviceProviderIds == value)
                    return;
                _serviceProviderIds = value;

                OnPropertyChanged(() => ServiceProviderIds);
            }
        }

        private string _serviceDesignId;
        public string ServiceDesignId
        {
            get
            {
                return _serviceDesignId;
            }
            set
            {
                if (_serviceDesignId == value)
                    return;

                _serviceDesignId = value;

                OnPropertyChanged(() => ServiceDesignId);
            }
        }

        private string _serviceInstanceId;
        public string ServiceInstanceId
        {
            get
            {
                return _serviceInstanceId;
            }
            set
            {
                if (_serviceInstanceId == value)
                    return;

                _serviceInstanceId = value;

                OnPropertyChanged(() => ServiceInstanceId);
            }
        }

        private string _mmsi;
        public string Mmsi
        {
            get
            {
                return _mmsi;
            }
            set
            {
                if (_mmsi == value)
                    return;
                _mmsi = value;

                OnPropertyChanged(() => Mmsi);
            }
        }

        private string _imo;
        public string Imo
        {
            get
            {
                return _imo;
            }
            set
            {
                if (_imo == value)
                    return;
                _imo = value;

                OnPropertyChanged(() => Imo);
            }
        }

        private string _serviceType;
        public string ServiceType
        {
            get
            {
                return _serviceType;
            }
            set
            {
                if (_serviceType == value)
                    return;
                _serviceType = value;

                OnPropertyChanged(() => ServiceType);
            }
        }

        private string _serviceStatus;
        public string ServiceStatus
        {
            get
            {
                return _serviceStatus;
            }
            set
            {
                if (_serviceStatus == value)
                    return;
                _serviceStatus = value;

                OnPropertyChanged(() => ServiceStatus);
            }
        }

        private string _keywords;
        public string Keywords
        {
            get
            {
                return _keywords;
            }
            set
            {
                if (_keywords == value)
                    return;

                _keywords = value;

                OnPropertyChanged(() => Keywords);
            }
        }

        private string _freeText;
        public string FreeText
        {
            get
            {
                return _freeText;
            }
            set
            {
                if (_freeText == value)
                    return;
                _freeText = value;

                OnPropertyChanged(() => FreeText);
            }
        }

        private ObservableCollection<FindServiceResponseObject> _services;
        public ObservableCollection<FindServiceResponseObject> Services
        {
            get
            {
                return _services;
            }
            set
            {
                if (_services == value)
                    return;

                _services = value;

                OnPropertyChanged(() => Services);
            }
        }

        private FindServiceResponseObject _selectedService;
        public FindServiceResponseObject SelectedService
        {
            get
            {
                return _selectedService;
            }
            set
            {
                if (_selectedService == value)
                    return;

                _selectedService = value;

                if (_selectedService != null)
                {
                    Text = _selectedService.InstanceAsXml.Content;
                    _settingsService.SelectedServiceUrl = _selectedService.EndpointUri;
                    _settingsService.SelectedServiceOrg = _selectedService.OrganizationId;
                }
                else
                {
                    _settingsService.SelectedServiceUrl = string.Empty;
                    _settingsService.SelectedServiceOrg = string.Empty;
                }

                OnPropertyChanged(() => SelectedService);
                UploadCommand.RaiseCanExecuteChanged();
                //GetCommand.RaiseCanExecuteChanged();
                GetSummaryCommand.RaiseCanExecuteChanged();
                SubscribeCommand.RaiseCanExecuteChanged();
                DeleteSubscriberCommand.RaiseCanExecuteChanged();
                GetSubscriptionsCommand.RaiseCanExecuteChanged();
                GetEncryptionKeyCommand.RaiseCanExecuteChanged();
            }
        }

        private GetSummaryResponseObject _summaryResponseObject;

        public GetSummaryResponseObject SummaryResponseObject
        {
            get
            {
                return _summaryResponseObject;
            }
            set
            {
                if (_summaryResponseObject == value) return;
                _summaryResponseObject = value;

                OnPropertyChanged(() => SummaryResponseObject);
            }
        }

        private ObservableCollection<SummaryObject> _summaryObjects;
        public ObservableCollection<SummaryObject> SummaryObjects
        {
            get
            {
                return _summaryObjects;
            }

            set
            {
                if (_summaryObjects == value)
                    return;

                _summaryObjects = value;

                OnPropertyChanged(() => SummaryObjects);
            }
        }

        private SummaryObject _summaryObject;
        public SummaryObject SummaryObject
        {
            get
            {
                return _summaryObject;
            }

            set
            {
                if (_summaryObject == value)
                    return;
                _summaryObject = value;

                OnPropertyChanged(() => SummaryObject);
                GetCommand.RaiseCanExecuteChanged();
                GetEncryptionKeyCommand.RaiseCanExecuteChanged();
            }
        }


        private string _text;
        public string Text
        {
            get
            {
                return _text;
            }
            set
            {
                if (_text == value)
                    return;

                _text = value;

                OnPropertyChanged(() => Text);
            }
        }

        #endregion

        #region commands
        private ICommand _findServicesCommand;
        public ICommand FindServicesCommand
        {
            get
            {
                return _findServicesCommand ??
                    (_findServicesCommand = new DelegateCommand(ExecuteFindServicesCommand, CanExecuteFindServicesCommand));
            }
        }

        public bool CanExecuteFindServicesCommand(object parameter)
        {
            return true;
        }

        public async void ExecuteFindServicesCommand(object parameter)
        {
            try
            {
                var query = new StringBuilder();
                if (!string.IsNullOrEmpty(Keywords))
                    AddToQuery("keywords", Keywords, ref query);
                if (!string.IsNullOrEmpty(ServiceDesignId))
                    AddToQuery("designId", ServiceDesignId, ref query);
                if (!string.IsNullOrEmpty(UnloCode))
                    AddToQuery("unlocode", UnloCode, ref query);
                if (!string.IsNullOrEmpty(ServiceInstanceId))
                    AddToQuery("instanceId", ServiceInstanceId, ref query);
                if (!string.IsNullOrEmpty(ServiceType))
                    AddToQuery("serviceType", ServiceType, ref query);
                if (!string.IsNullOrEmpty(ServiceStatus))
                    AddToQuery("status", ServiceStatus, ref query);
                if (!string.IsNullOrEmpty(Mmsi))
                    AddToQuery("mmsi", Mmsi, ref query);
                if (!string.IsNullOrEmpty(Imo))
                    AddToQuery("imo", Imo, ref query);
                if (!string.IsNullOrEmpty(ServiceProviderIds))
                    AddToQuery("organizationId", ServiceProviderIds, ref query);

                var response = await _secomService.FindService(query.ToString(), string.Empty, 0, 1000);
                Services = new ObservableCollection<FindServiceResponseObject>(response);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private DelegateCommand _uploadCommand;
        public DelegateCommand UploadCommand
        {
            get
            {
                return _uploadCommand ??
                    (_uploadCommand = new DelegateCommand(ExecuteUploadCommand, CanExecuteUploadCommand));
            }
        }

        private bool CanExecuteUploadCommand(object obj)
        {
            return SelectedService != null;
        }

        private async void ExecuteUploadCommand(object obj)
        {
            var dlg = (NewMessageDialog)_serviceProvider.GetService(typeof(NewMessageDialog));
            if (dlg.ShowDialog() == true)
            {
                var containerType = (ContainerTypeEnum)dlg.ViewModel.ContainerType;
                var myCert = new X509Certificate2(_settingsService.CertificateFile, _settingsService.CertificatePassword, X509KeyStorageFlags.Exportable);
                byte[] data;

                var serializeOptions = new JsonSerializerOptions
                {
                    PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
                };

                // If exchange folder creata tar archive
                //if (dlg.ViewModel.DataType == (int)DataTypeEnum.S100_ExchangeSet)
                //{
                //    data = _compressionService.Archive(dlg.ViewModel.File);
                //}
                //else
                //{
                //    data = File.ReadAllBytes(dlg.ViewModel.File);
                //}

                data = File.ReadAllBytes(dlg.ViewModel.File);

                // Keep original data for signing
                byte[] originalData = new byte[data.Length];
                data.CopyTo(originalData, 0);

                // If compressionFlag, gzip data
                if (dlg.ViewModel.CompressionFlag)
                {
                    data = _compressionService.Compress(data);
                }
                var envCert = new X509Certificate2(_settingsService.EnvelopeCertificateFile, _settingsService.EnvelopeCertificatePassword, X509KeyStorageFlags.Exportable);
                // If data protection encrypt data
                if (dlg.ViewModel.DataProtection)
                {
                    data = _encryptionService.Encrypt(data, out var key, out var iv);

                    var receiverPublicKey = new X509Certificate2(dlg.ViewModel.PublicKey);
                    var senderKey = new X509Certificate2(_settingsService.CertificateFile, _settingsService.CertificatePassword);
                    var encryptedKey = _encryptionService.EncryptWithDerivedKey(key, senderKey, receiverPublicKey, iv);

                    // Send key
                    var keyEnvelope = new EnvelopeKeyObject
                    {
                        EncryptionKey = encryptedKey,
                        IV = iv,
                        TransactionIdentifier = dlg.ViewModel.TransactionId,
                        DigitalSignatureValue = _signatureService.GenerateSignature(key, myCert),
                        EnvelopeSignatureCertificate = _certificateService.GetBase64SingleLine(envCert),
                        EnvelopeRootCertificateThumbprint = _certificateService.GetRootCertificate(envCert).Thumbprint,
                        EnvelopeSignatureTime = DateTime.UtcNow,                      
                    };

                    var envelopeKeySignature = _signatureService.GenerateSignature(keyEnvelope, envCert);
                    var encryptionKey = new EncryptionKeyObject
                    {
                        Envelope = keyEnvelope,
                        EnvelopeSignature = envelopeKeySignature.DigitalSignature
                    };

                    var keyStringContent = JsonSerializer.Serialize(encryptionKey, serializeOptions);
                    File.WriteAllText($@"c:\SECOM_Files\Debug\EncryptionKey-{dlg.ViewModel.TransactionId:n}.txt", keyStringContent);

                    var keyRequest = new CallServiceRequestDto
                    {
                        EndpointMethod = $"{SelectedService.EndpointUri}/v1/EncryptionKey/",
                        RequestType = "POST",
                        Body = keyStringContent
                    };

                    var keyResponse = await _secomService.CallService(keyRequest);
                    if ((int)keyResponse.StatusCode < 200 ||
                        (int)keyResponse.StatusCode > 299)
                    {
                        MessageBox.Show($"Unable to upload. {keyResponse.StatusCode} - {keyResponse.Body}");
                    }
                }

                if (dlg.ViewModel.UploadLink)
                {
                    var envelopeObject = new EnvelopeLinkObject
                    {
                        //AckEndpoint = ackEndpointUrl,
                        AckRequest = (AckRequestEnum)dlg.ViewModel.Acknowledgement,
                        ContainerType = (ContainerTypeEnum)dlg.ViewModel.ContainerType,
                        DataProductType = (Secom_DataProductTypeEnum)dlg.ViewModel.DataProductType,
                        FromSubscription = false,
                        Size = data.Length,
                        TimeToLive = DateTime.Now.AddMinutes(15),
                        TransactionIdentifier = dlg.ViewModel.TransactionId,
                        ExchangeMetadata = new ExchangeMetadata
                        {
                            CompressionFlag = dlg.ViewModel.CompressionFlag,
                            DataProtection = dlg.ViewModel.DataProtection,
                            DigitalSignatureReference = dlg.ViewModel.DigitalSignatureReference,
                            ProtectionScheme = dlg.ViewModel.ProtectionScheme,
                            DigitalSignatureValue = _signatureService.GenerateSignature(originalData, myCert)
                        },
                        EnvelopeSignatureCertificate = _certificateService.GetBase64SingleLine(myCert),
                        EnvelopeRootCertificateThumbprint = _certificateService.GetRootCertificate(myCert).Thumbprint,
                        EnvelopeSignatureTime = DateTime.UtcNow,
                    };

                    var envelopeSignature = _signatureService.GenerateSignature(envelopeObject, envCert);
                    var uploadLinkObj = new UploadLinkObject
                    {
                        Envelope = envelopeObject,
                        EnvelopeSignature = envelopeSignature.DigitalSignature
                    };

                    var stringContent = JsonSerializer.Serialize(uploadLinkObj, serializeOptions).Replace(Environment.NewLine, "");

                    File.WriteAllText($@"c:\SECOM_Files\Debug\UploadLinkObj-{dlg.ViewModel.TransactionId:n}.txt", stringContent);
                    var request = new CallServiceRequestDto
                    {
                        //EndpointMethod = $"{SelectedService.EndpointUri}/v1/{payloadType.ToString().ToLower()}/link/",
                        EndpointMethod = $"{SelectedService.EndpointUri}/v1/object/link/",
                        RequestType = "POST",
                        Body = stringContent
                    };

                    // Store outgoing message
                    var saved = _sentMessageService.Save(uploadLinkObj, data, SelectedService.InstanceId, envelopeObject.TimeToLive);

                    var response = await _secomService.CallService(request);
                    if ((int)response.StatusCode < 200 ||
                        (int)response.StatusCode > 299)
                    {
                        MessageBox.Show($"Unable to upload. {response.StatusCode} - {response.Body}");
                    }
                    else
                    {
                        MessageBox.Show($"Successfully uploaded message {dlg.ViewModel.TransactionId:n} to {SelectedService.InstanceId}.");
                    }
                }
                else
                {
                    var envelopeObject = new EnvelopeUploadObject
                    {
                        AckRequest = (AckRequestEnum)dlg.ViewModel.Acknowledgement,
                        Data = data,
                        ContainerType = (ContainerTypeEnum)dlg.ViewModel.ContainerType,
                        DataProductType = (Secom_DataProductTypeEnum)dlg.ViewModel.DataProductType,
                        FromSubscription = false,
                        TransactionIdentifier = dlg.ViewModel.TransactionId,
                        ExchangeMetadata = new ExchangeMetadata
                        {
                            CompressionFlag = dlg.ViewModel.CompressionFlag,
                            DataProtection = dlg.ViewModel.DataProtection,
                            DigitalSignatureReference = dlg.ViewModel.DigitalSignatureReference,
                            ProtectionScheme = dlg.ViewModel.ProtectionScheme,
                            DigitalSignatureValue = _signatureService.GenerateSignature(originalData, myCert)
                        },
                        EnvelopeSignatureCertificate = _certificateService.GetBase64SingleLine(myCert),
                        EnvelopeRootCertificateThumbprint = _certificateService.GetRootCertificate(myCert).Thumbprint,
                        EnvelopeSignatureTime = DateTime.UtcNow,
                    };

                    var envelopeSignature = _signatureService.GenerateSignature(envelopeObject, envCert);
                    var uploadObj = new UploadObject
                    {
                        Envelope = envelopeObject,
                        EnvelopeSignature = envelopeSignature.DigitalSignature
                    };

                    var stringContent = JsonSerializer.Serialize(uploadObj, serializeOptions).Replace(Environment.NewLine, "");

                    File.WriteAllText($@"c:\SECOM_Files\Debug\UploadObject-{dlg.ViewModel.TransactionId:n}.txt", stringContent);
                    var request = new CallServiceRequestDto
                    {
                        //EndpointMethod = $"{SelectedService.EndpointUri}/v1/{payloadType.ToString().ToLower()}/",
                        EndpointMethod = $"{SelectedService.EndpointUri}/v1/object/",
                        RequestType = "POST",
                        Body = stringContent
                    };

                    var response = await _secomService.CallService(request);
                    if ((int)response.StatusCode < 200 ||
                        (int)response.StatusCode > 299)
                    {
                        MessageBox.Show($"Unable to upload. {response.StatusCode} - {response.Body}");
                    }
                    else
                    {
                        MessageBox.Show($"Successfully uploaded message {dlg.ViewModel.TransactionId:n} to {SelectedService.InstanceId}.");
                        //Store outgoing message
                        var saved = _sentMessageService.Save(uploadObj, SelectedService.InstanceId);
                    }
                }
            }
        }

        private DelegateCommand _getEncryptionKeyCommand;
        public DelegateCommand GetEncryptionKeyCommand
        {
            get
            {
                return _getEncryptionKeyCommand ?? (_getEncryptionKeyCommand = new DelegateCommand(ExcecuteGetEncryptionKeyCommand, CanExecuteGetEncryptionKeyCommand));
            }
        }

        private bool CanExecuteGetEncryptionKeyCommand(object obj)
        {
            return SelectedService != null && SummaryObject != null && SummaryObject.DataProtection;
        }
        private async void ExcecuteGetEncryptionKeyCommand(object obj)
        {
            if (!SummaryObject.DataProtection)
            {
                MessageBox.Show("Selected message is not encrypted!");
                return;
            }

            //Check if key already has been previously downloaded
            var keyExists = await _secomService.GetEncryptionKey(new Guid(SummaryObject.DataReference));
            if(keyExists != null)
            {
                MessageBox.Show($"Encryption key for data {SummaryObject.DataReference} already downloaded.");
                return;
            }

            var myCert = new X509Certificate2(_settingsService.CertificateFile, _settingsService.CertificatePassword, X509KeyStorageFlags.Exportable);
            var envCert = new X509Certificate2(_settingsService.EnvelopeCertificateFile, _settingsService.EnvelopeCertificatePassword, X509KeyStorageFlags.Exportable);
            var myPublicKey = _certificateService.GetBase64SingleLine(myCert);
            var endpoint = SelectedService.EndpointUri + "/v1/EncryptionKey/request";

            var keyEnvelope = new EnvelopeKeyRequestObject
            {
                DataReference = new Guid(SummaryObject.DataReference),
                PublicCertificate = myPublicKey,
                EnvelopeSignatureCertificate = _certificateService.GetBase64SingleLine(envCert),
                EnvelopeSignatureTime = DateTime.UtcNow
            };
            var envelopeSignature = _signatureService.GenerateSignature(keyEnvelope, envCert);

            var body = new EncryptionKeyRequestObject
            {
                Envelope = keyEnvelope,
                EnvelopeSignature = envelopeSignature.DigitalSignature
            };

            var serializeOptions = new JsonSerializerOptions
            {
                PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
            };
            var stringContent = JsonSerializer.Serialize(body, serializeOptions).Replace(Environment.NewLine, "");
            File.WriteAllText($@"c:\SECOM_Files\Debug\GetEncryptionKeyRequest.txt", stringContent);
            var request = new CallServiceRequestDto
            {
                EndpointMethod = endpoint,
                RequestType = "POST",
                Body = stringContent
            };

            var response = await _secomService.CallService(request);
            if ((int)response.StatusCode < 200 ||
                (int)response.StatusCode > 299)
            {
                MessageBox.Show($"Failed get request. {response.StatusCode} - {response.Body}");
            }
            else
            {
                MessageBox.Show($"Successful request to {SelectedService.EndpointUri}");
            }
        }

        private DelegateCommand _getSummaryCommand;

        public DelegateCommand GetSummaryCommand
        {
            get
            {
                return _getSummaryCommand ?? (_getSummaryCommand = new DelegateCommand(ExecuteGetSummaryCommand, CanExecuteGetSummaryCommand));
            }
        }

        private bool CanExecuteGetSummaryCommand(object obj)
        {
            return SelectedService != null;
        }

        private async void ExecuteGetSummaryCommand(object obj)
        {
            var dlg = (GetSummaryDialog)_serviceProvider.GetService(typeof(GetSummaryDialog));
            var serializeOptions = new JsonSerializerOptions
            {
                PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
            };

            if (dlg.ShowDialog() == true)
            {
                var dataRef = dlg.ViewModel.DataReference;
                var containerType = (ContainerTypeEnum?)dlg.ViewModel.ContainerType;
                var dataProductType = (Secom_DataProductTypeEnum?)dlg.ViewModel.DataProductType;
                var productVersion = dlg.ViewModel.ProductVersion;
                var geo = dlg.ViewModel.Geometry;
                var unlocode = dlg.ViewModel.Unlocode;
                var fromTime = dlg.ViewModel.FromTime;
                var toTime = dlg.ViewModel.ToTime;
                var page = dlg.ViewModel.Page;
                var pageSize = dlg.ViewModel.PageSize;

                string query = string.Empty;

                if (!string.IsNullOrEmpty(dataRef)) query += $"&dataReference={dataRef}";
                if (containerType.HasValue) query += $"&containerType={containerType.Value}";
                if (!string.IsNullOrEmpty(geo)) query += $"&geometry={geo}";
                if (!string.IsNullOrEmpty(unlocode)) query += $"&unlocode={unlocode}";
                if (fromTime.HasValue) query += $"&validFrom={fromTime.Value}";
                if (toTime.HasValue) query += $"&validTo={toTime.Value}";
                if (dataProductType.HasValue) query += $"&dataProductType={dataProductType.Value}";
                if (!string.IsNullOrEmpty(productVersion)) query += $"productVersion={productVersion}";


                if (query.Contains('&'))
                {
                    query += $"&page={page}&pageSize={pageSize}";
                }
                else
                {
                    query += $"page={page}&pageSize={pageSize}";
                }

                File.WriteAllText($@"c:\SECOM_Files\Debug\GetSummaryQuery.txt", query);
                //var endpoint = SelectedService.EndpointUri + $"/v1/{dataType.ToString().ToLower()}/summary?";
                var endpoint = SelectedService.EndpointUri + $"/v1/object/summary?";
                var request = new CallServiceRequestDto
                {
                    EndpointMethod = endpoint + query,
                    RequestType = "GET"
                };

                var response = await _secomService.CallService(request);
                if ((int)response.StatusCode < 200 ||
                    (int)response.StatusCode > 299)
                {
                    MessageBox.Show($"Failed get summary request. {response.StatusCode} - {response.Body}");
                }
                else
                {

                    JsonSerializerOptions options = new(JsonSerializerDefaults.Web);
                    SummaryResponseObject = JsonSerializer.Deserialize<GetSummaryResponseObject>(response.Body, options);

                    if (SummaryResponseObject != null && SummaryResponseObject.InformationSummaryObject != null)
                    {
                        SummaryObjects = new ObservableCollection<SummaryObject>();
                        foreach (var responseObject in SummaryResponseObject.InformationSummaryObject)
                        {
                            SummaryObjects.Add(responseObject);
                        }
                    }

                    var stringContent = JsonSerializer.Serialize(SummaryResponseObject, serializeOptions).Replace(Environment.NewLine, "");
                    File.WriteAllText($@"c:\SECOM_Files\Debug\GetSummaryResponse.txt", stringContent);
                }
            }
        }

        private DelegateCommand _getCommand;
        public DelegateCommand GetCommand
        {
            get
            {
                return _getCommand ?? (_getCommand = new DelegateCommand(ExecuteGetCommand, CanExecuteGetCommand));
            }
        }

        private bool CanExecuteGetCommand(object obj)
        {
            return SummaryObject != null;
        }

        private async void ExecuteGetCommand(object obj)
        {
            string query = string.Empty;
            var serializeOptions = new JsonSerializerOptions
            {
                PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
            };

            var dataRef = SummaryObject.DataReference;
            if (string.IsNullOrEmpty(dataRef))
            {
                MessageBox.Show("DataReference is missing!");
                return;
            }
            else
            {
                query += $"&dataReference={dataRef}";
            }
            var dataType = SummaryObject.ContainerType;
            //var endpoint = SelectedService.EndpointUri + $"/v1/{dataType.ToString().ToLower()}?";
            var endpoint = SelectedService.EndpointUri + "/v1/object?";

            if (query.Contains('&'))
            {
                query += $"&page={1}&pageSize={3}";
            }
            else
            {
                query += $"page={1}&pageSize={3}";
            }

            File.WriteAllText($@"c:\SECOM_Files\Debug\GetQuery-{dataRef:n}.txt", query);

            var request = new CallServiceRequestDto
            {
                EndpointMethod = endpoint + query,
                RequestType = "GET"
            };

            var response = await _secomService.CallService(request);
            if ((int)response.StatusCode < 200 ||
                (int)response.StatusCode > 299)
            {
                MessageBox.Show($"Failed get request. {response.StatusCode} - {response.Body}");
            }
            else
            {
                //TODO: Store messages somewhere

                JsonSerializerOptions options = new(JsonSerializerDefaults.Web);
                var responseObjects = JsonSerializer.Deserialize<GetResponseObject>(response.Body, options);

                var stringContent = JsonSerializer.Serialize(responseObjects, serializeOptions).Replace(Environment.NewLine, "");
                File.WriteAllText($@"c:\SECOM_Files\Debug\GetResponse-{dataRef:n}.txt", stringContent);

                if (responseObjects != null && responseObjects.DataResponseObject != null)
                {
                    foreach (var responseObject in responseObjects.DataResponseObject)
                    {
                        var shouldSendOperationalAck = responseObject.AckRequest == AckRequestEnum.Delivered_and_Opened_ACK_requested || responseObject.AckRequest == AckRequestEnum.Opened_ACK_requested;
                        bool nackResult = false;
                        byte[] originalData = responseObject.Data;
                        var digsig = new DigitalSignatureValueObject
                        {
                            DigitalSignature = responseObject.ExchangeMetadata.DigitalSignatureValue.DigitalSignature,
                            PublicCertificate = responseObject.ExchangeMetadata.DigitalSignatureValue.PublicCertificate,
                            PublicRootCertificateThumbprint = responseObject.ExchangeMetadata.DigitalSignatureValue.PublicRootCertificateThumbprint
                        };

                        if(responseObject.ExchangeMetadata.DataProtection)
                        {
                            try
                            {
                                var encryptionKey = await _secomService.GetEncryptionKey(new Guid(SummaryObject.DataReference));
                                if (encryptionKey == null)
                                {
                                    MessageBox.Show($"Data with id {dataRef} is encrypted! No related encryption key found!");
                                    return;
                                }
                                var senderPublicKey = new X509Certificate2(Convert.FromBase64String(digsig.PublicCertificate));
                                var receiverKey = new X509Certificate2(_settingsService.CertificateFile, _settingsService.CertificatePassword);
                                var decryptedKey = _encryptionService.DecryptWithDerivedKey(encryptionKey.Key, receiverKey, senderPublicKey, encryptionKey.IV);
                                originalData = _encryptionService.Decrypt(originalData, decryptedKey, encryptionKey.IV);
                            }
                            catch(Exception ex)
                            {
                                MessageBox.Show($"Failed to decrypt message {SummaryObject.DataReference}, notify sender.");
                                if(shouldSendOperationalAck)
                                    nackResult = await SendOperationalAck(NackTypeEnum.FailedDecryption);
                            }
                        }

                        if (responseObject.ExchangeMetadata.CompressionFlag)
                        {
                            try
                            {
                                originalData = _compressionService.Decompress(responseObject.Data);
                            }
                            catch(Exception ex)
                            {
                                MessageBox.Show($"Failed to decompress message {SummaryObject.DataReference}, notify sender.");
                                if (shouldSendOperationalAck)
                                    nackResult = await SendOperationalAck(NackTypeEnum.FailedDecompression);
                            }                           
                        }

                        if (_signatureService.Verify(digsig, originalData, out string result))
                        {
                            bool ackWasSent = false;
                            switch (dataType)
                            {   
                                case ContainerTypeEnum.S100_EXCHANGESET:
                                    MessageBox.Show(ViewExchangeSet(originalData));
                                    if(shouldSendOperationalAck)
                                    {
                                        ackWasSent = await SendOperationalAck();
                                    }
                                    break;
                                case ContainerTypeEnum.NONE:
                                    MessageBox.Show(ViewText(originalData));
                                    if (shouldSendOperationalAck)
                                    {
                                        ackWasSent = await SendOperationalAck();
                                    }
                                    break;
                                default:
                                    MessageBox.Show(ViewXml(originalData));
                                    if (shouldSendOperationalAck)
                                    {
                                        ackWasSent = await SendOperationalAck();
                                    }
                                    break;
                            }
                            if (ackWasSent)
                                MessageBox.Show($"Successfully sent operationl ack for {dataRef}.");
                            else
                            {
                                if (shouldSendOperationalAck)
                                    MessageBox.Show($"Failed sending operational ack for {dataRef}.");
                            }
                        }
                        else
                        {
                            MessageBox.Show($"Signature verification failed!");
                            if (shouldSendOperationalAck)
                                nackResult = await SendOperationalAck(NackTypeEnum.FailedSignatureVerification);
                        }
                    }
                }
            }
        }

        private DelegateCommand _subscribeCommand;
        public DelegateCommand SubscribeCommand
        {
            get
            {
                return _subscribeCommand ?? (_subscribeCommand = new DelegateCommand(ExecuteSubscribeCommand, CanExecuteSubscribeCommand));
            }
        }

        private bool CanExecuteSubscribeCommand(object obj)
        {
            return SelectedService != null;
        }

        private async void ExecuteSubscribeCommand(object obj)
        {
        }

        private DelegateCommand _deleteSubscriberCommand;
        public DelegateCommand DeleteSubscriberCommand
        {
            get
            {
                return _deleteSubscriberCommand ?? (_deleteSubscriberCommand = new DelegateCommand(ExecuteDeleteSubscriberCommand, CanExecuteDeleteSubscriberCommand));
            }
        }

        private bool CanExecuteDeleteSubscriberCommand(object obj)
        {
            return SelectedService != null;
        }

        private async void ExecuteDeleteSubscriberCommand(object obj)
        {
        }


        private DelegateCommand _getSubscriptionsCommand;
        public DelegateCommand GetSubscriptionsCommand
        {
            get
            {
                return _getSubscriptionsCommand ?? (_getSubscriptionsCommand = new DelegateCommand(ExecuteGetSubscriptionsCommand, CanExecuteGetSubscriptionsCommand));
            }
        }

        private bool CanExecuteGetSubscriptionsCommand(object obj)
        {
            return SelectedService != null;
        }

        private async void ExecuteGetSubscriptionsCommand(object obj)
        {
        }
        #endregion

        #region privateMethods
        private void AddToQuery(string key, string value, ref StringBuilder sb)
        {
            if (sb.Length > 0)
                sb.Append(string.Format(" AND {0}:{1}", key, value.Replace(":", "\\:")));
            else
                sb.Append(string.Format("{0}:{1}", key, value.Replace(":", "\\:")));
        }

        private async System.Threading.Tasks.Task<bool> SendOperationalAck(NackTypeEnum? nackType = null)
        {
            var serializeOptions = new JsonSerializerOptions
            {
                PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
            };

            var envCert = new X509Certificate2(_settingsService.EnvelopeCertificateFile, _settingsService.EnvelopeCertificatePassword, X509KeyStorageFlags.Exportable);

            var ackData = new EnvelopeAckObject
            {
                CreatedAt = DateTime.Now,
                TransactionIdentifier = new Guid(SummaryObject.DataReference),
                EnvelopeCertificate = _certificateService.GetBase64SingleLine(envCert),
                EnvelopeRootCertificateThumbprint = _certificateService.GetRootCertificate(envCert).Thumbprint,
                NackType = nackType,
                AckType = AckTypeEnum.Opened,
                EnvelopeSignatureTime = DateTime.Now
            };
            var ackSignature = _signatureService.GenerateSignature(ackData, envCert);
            var ackObject = new AcknowledgementObject
            {
                Envelope = ackData,
                DigitalSignature = ackSignature.DigitalSignature
            };
            var ackStringContent = JsonSerializer.Serialize(ackObject, serializeOptions);
            File.WriteAllText($@"c:\SECOM_Files\Debug\Ack_operational-{SummaryObject.DataReference:n}.txt", ackStringContent);
            var request = new CallServiceRequestDto
            {
                EndpointMethod = SelectedService.EndpointUri + $"/v1/Acknowledgement",
                RequestType = "POST",
                Body = ackStringContent,
            };
            var response = await _secomService.CallService(request);
            if ((int)response.StatusCode < 200 || (int)response.StatusCode > 299)
            {
                return false;
            }
            else
            {
                //Set AckDelivered 
                //await _secomService.SaveAcknowledgement(ackObject);
                return true;
            }
        }

        private string ViewExchangeSet(byte[] data)
        {
            return $"Data is an exchange set";
        }

        private string ViewText(byte[] data)
        {
            return Encoding.UTF8.GetString(data);
        }

        private string ViewXml(byte[] data)
        {
            var xml = Encoding.UTF8.GetString(data);
            var stringBuilder = new StringBuilder();

            var element = XElement.Parse(xml);

            var settings = new XmlWriterSettings();
            settings.OmitXmlDeclaration = true;
            settings.Indent = true;
            settings.NewLineOnAttributes = true;
            
            using (var xmlWriter = XmlWriter.Create(stringBuilder, settings))
            {
                element.Save(xmlWriter);
            }

            return stringBuilder.ToString();

        }

        #endregion
    }
}