﻿using SECOM.Common.Models.Private;
using SECOM.Module.Ui.Services;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Timers;
using System.Windows;

namespace SECOM.Module.Ui.ViewModels
{
    public sealed class MainViewModel : ViewModelBase, IDisposable
    {
        private Timer _notificationTimer;
        private readonly ISecomService _secomService;
        private readonly SettingsService _settingsService;
        private readonly NotificationService _notificationService;

        public MainViewModel(ISecomService secomService,
            SettingsService settingsService,
            NotificationService notificationService)
        {
            _secomService = secomService;
            _settingsService = settingsService;
            _notificationService = notificationService;

            settingsService.SettingsChanged += (sender, e) =>
            {
                Notifications.Clear();
                OnPropertyChanged(() => InstanceDisplayName);
                OnPropertyChanged(() => Environment);
            };

            _notificationService.NewNotification += Instance_NewNotification;

            _notificationTimer = new Timer(10000);
            _notificationTimer.AutoReset = true;
            _notificationTimer.Elapsed += _notificationTimer_Elapsed;
            _notificationTimer.Start();
        }

        public string Environment
        {
            get { return _settingsService.Environment; }
        }
        public string InstanceDisplayName
        {
            get { return _settingsService.InstanceDisplayName; }
        }

        private void _notificationTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            try
            {
                if (!string.IsNullOrEmpty(_settingsService.InstanceName))
                {
                    List<NotificationDto> notifications = new List<NotificationDto>();

                    var secomNotifications = _secomService.GetNotifications().Result;
                    if (secomNotifications != null && secomNotifications.Count > 0)
                        notifications.AddRange(secomNotifications);

                    if (notifications != null)
                    {
                        foreach (var notification in notifications)
                        {
                            Instance_NewNotification(sender, notification);
                        }
                    }
                }
            }
            catch(AggregateException ex)
            {
                var msg = $"{ex.Message}, ";
                foreach(var innner in ex.InnerExceptions)
                {
                    msg += $"{innner.Message}, ";
                }
                throw ex;
            }
        }

        private ObservableCollection<NotificationDto> _notifications = new ObservableCollection<NotificationDto>();
        public ObservableCollection<NotificationDto> Notifications
        {
            get
            {
                return _notifications;
            }
            set
            {
                if (_notifications == value)
                    return;

                _notifications = value;

                OnPropertyChanged(() => Notifications);
            }
        }

        private NotificationDto _selectedNotification;
        public NotificationDto SelectedNotification
        {
            get
            {
                return _selectedNotification;
            }
            set
            {
                if (_selectedNotification == value)
                    return;

                _selectedNotification = value;

                OnPropertyChanged(() => SelectedNotification);
            }
        }

        private void Instance_NewNotification(object sender, NotificationDto e)
        {
            Application.Current.Dispatcher.Invoke(new Action(() => { Notifications.Add(e); }));
        }

        public void Dispose()
        {
            _notificationTimer.Dispose();
            Dispose();
        }
    }
}