﻿using Microsoft.Win32;
using SECOM.Module.Ui.Models;
using SECOM.Module.Ui.Utils;
using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Windows;
using System.Windows.Input;

namespace SECOM.Module.Ui.ViewModels
{
    public class NewMessageDialogViewModel : ViewModelBase
    {
        public NewMessageDialogViewModel()
        {

            Acknowledgements = new List<ComboBoxData>();
            Acknowledgements.Add(new ComboBoxData { Text = "No ack requested", Value = 0 });
            Acknowledgements.Add(new ComboBoxData { Text = "Technical ack", Value = 1 });
            Acknowledgements.Add(new ComboBoxData { Text = "Operational ack", Value = 2 });
            Acknowledgements.Add(new ComboBoxData { Text = "Operational and technical ack", Value = 3 });

            NackTypes = new List<ComboBoxData>();
            NackTypes.Add(new ComboBoxData { Text = "XMLSchemaValidationError", Value = 0});
            NackTypes.Add(new ComboBoxData { Text = "UnknownDataTypeOrVersion", Value = 1});
            NackTypes.Add(new ComboBoxData { Text = "FailedSignatureVerification", Value = 2});
            NackTypes.Add(new ComboBoxData { Text = "FailedDecryption", Value = 3});
            NackTypes.Add(new ComboBoxData { Text = "FailedDecompression", Value = 4});

            ContainerTypes = new List<ComboBoxData>();
            ContainerTypes.Add(new ComboBoxData { Text = "S100_DATASET", Value = 0 });
            ContainerTypes.Add(new ComboBoxData { Text = "S100_EXCHANGESET", Value = 1 });
            ContainerTypes.Add(new ComboBoxData { Text = "NONE", Value = 2 });

            DataProductTypes = new List<ComboBoxData>();
            DataProductTypes.Add(new ComboBoxData { Text = "S57", Value = 0 });
            DataProductTypes.Add(new ComboBoxData { Text = "S101", Value = 1 });
            DataProductTypes.Add(new ComboBoxData { Text = "S102", Value = 2 });
            DataProductTypes.Add(new ComboBoxData { Text = "S104", Value = 3 });
            DataProductTypes.Add(new ComboBoxData { Text = "S111", Value = 4 });
            DataProductTypes.Add(new ComboBoxData { Text = "S122", Value = 5 });
            DataProductTypes.Add(new ComboBoxData { Text = "S123", Value = 6 });
            DataProductTypes.Add(new ComboBoxData { Text = "S124", Value = 7 });
            DataProductTypes.Add(new ComboBoxData { Text = "S125", Value = 8 });
            DataProductTypes.Add(new ComboBoxData { Text = "S126", Value = 9 });
            DataProductTypes.Add(new ComboBoxData { Text = "S127", Value = 10 });
            DataProductTypes.Add(new ComboBoxData { Text = "S128", Value = 11 });
            DataProductTypes.Add(new ComboBoxData { Text = "S129", Value = 12 });
            DataProductTypes.Add(new ComboBoxData { Text = "S210", Value = 13 });
            DataProductTypes.Add(new ComboBoxData { Text = "S211", Value = 14 });
            DataProductTypes.Add(new ComboBoxData { Text = "S212", Value = 15 });
            DataProductTypes.Add(new ComboBoxData { Text = "S401", Value = 16 });
            DataProductTypes.Add(new ComboBoxData { Text = "S402", Value = 17 });
            DataProductTypes.Add(new ComboBoxData { Text = "S411", Value = 18 });
            DataProductTypes.Add(new ComboBoxData { Text = "S412", Value = 19 });
            DataProductTypes.Add(new ComboBoxData { Text = "S413", Value = 20 });
            DataProductTypes.Add(new ComboBoxData { Text = "S414", Value = 21 });
            DataProductTypes.Add(new ComboBoxData { Text = "S421", Value = 22 });
            DataProductTypes.Add(new ComboBoxData { Text = "RTZ", Value = 23 });
            DataProductTypes.Add(new ComboBoxData { Text = "EPC", Value = 24 });
            DataProductTypes.Add(new ComboBoxData { Text = "OTHER", Value = 25 });

            //ProdSpecVersions = new List<ComboBoxData>();
            //ProdSpecVersions.Add(new ComboBoxData { Text = "1.0.0", Value = 0 });
            //ProdSpecVersions.Add(new ComboBoxData { Text = "2.0.0", Value = 1 });
            //ProdSpecVersions.Add(new ComboBoxData { Text = "3.0.0", Value = 2 });
            //ProdSpecVersions.Add(new ComboBoxData { Text = "4.0.0", Value = 3 });
            //ProdSpecVersions.Add(new ComboBoxData { Text = "1.0", Value = 4 });
            //ProdSpecVersions.Add(new ComboBoxData { Text = "1.1", Value = 5 });
            //ProdSpecVersions.Add(new ComboBoxData { Text = "1.2", Value = 6 });

        }

        private Guid _transactionId = Guid.NewGuid();
        public Guid TransactionId
        {
            get
            {
                return _transactionId;
            }
            set
            {
                if (_transactionId == value)
                    return;
                _transactionId = value;

                OnPropertyChanged(() => TransactionId);
            }
        }

        private int _containerType = 0;
        public int ContainerType
        {
            get
            {
                return _containerType;
            }
            set
            {
                if (_containerType == value)
                    return;
                _containerType = value;

                OnPropertyChanged(() => ContainerType);
            }
        }

        private int _dataProducTypee;
        public int DataProductType
        {
            get
            {
                return _dataProducTypee;
            }
            set
            {
                if (_dataProducTypee == value)
                    return;
                _dataProducTypee = value;

                OnPropertyChanged(() => DataProductType);
            }
        }

        private string _unlocode;
        public string Unlocode
        {
            get
            {
                return _unlocode;
            }
            set
            {
                if (_unlocode == value)
                    return;
                _unlocode = value;

                OnPropertyChanged(() => Unlocode);
            }
        }

        private string _protectionScheme = "SECOM";
        public string ProtectionScheme
        {
            get
            {
                return _protectionScheme;
            }
            set
            {
                if (_protectionScheme == value)
                    return;
                _protectionScheme = value;

                OnPropertyChanged(() => ProtectionScheme);
            }
        }

        private string _digitalSignatureReference = "DSA";
        public string DigitalSignatureReference
        {
            get
            {
                return _digitalSignatureReference;
            }
            set
            {
                if (_digitalSignatureReference == value)
                    return;
                _digitalSignatureReference = value;

                OnPropertyChanged(() => DigitalSignatureReference);
            }
        }


        private string _file = "";
        public string File
        {
            get
            {
                return _file;
            }
            set
            {
                if (_file == value)
                    return;
                _file = value;

                OnPropertyChanged(() => File);
            }
        }

        private string _publicKey = "";
        public string PublicKey
        {
            get
            {
                return _publicKey;
            }
            set
            {
                if (_publicKey == value)
                    return;
                _publicKey = value;

                OnPropertyChanged(() => PublicKey);
            }
        }

        private int _acknowledgement = 0;
        public int Acknowledgement
        {
            get
            {
                return _acknowledgement;
            }
            set
            {
                if (_acknowledgement == value)
                    return;
                _acknowledgement = value;

                OnPropertyChanged(() => Acknowledgement);
            }
        }

        private bool _dataProtection = false;
        public bool DataProtection
        {
            get
            {
                return _dataProtection;
            }
            set
            {
                if (_dataProtection == value)
                    return;
                _dataProtection = value;

                OnPropertyChanged(() => DataProtection);
                ((DelegateCommand)BrowseForPublicKeyCommand).RaiseCanExecuteChanged();
            }
        }

        private bool _compressionFlag = false;
        public bool CompressionFlag
        {
            get
            {
                return _compressionFlag;
            }
            set
            {
                if (_compressionFlag == value)
                    return;
                _compressionFlag = value;

                OnPropertyChanged(() => CompressionFlag);
            }
        }


        private bool _uploadLink = false;
        public bool UploadLink
        {
            get
            {
                return _uploadLink;
            }
            set
            {
                if (_uploadLink == value)
                    return;
                _uploadLink = value;

                OnPropertyChanged(() => UploadLink);
            }
        }

        private bool _isExchangeSet = false;
        public bool IsExchangeSet
        {
            get
            {
                return _isExchangeSet;
            }
            set
            {
                if (_isExchangeSet == value)
                    return;
                _isExchangeSet = value;

                OnPropertyChanged(() => IsExchangeSet);
            }
        }

        public List<ComboBoxData> ContainerTypes { get; }
        public List<ComboBoxData> DataProductTypes { get; }
        public List<ComboBoxData> Classifications { get; }
        public List<ComboBoxData> Acknowledgements { get; }
        public List<ComboBoxData> NackTypes { get; }

        private ICommand _browseForFileCommand;
        public ICommand BrowseForFileCommand
        {
            get
            {
                return _browseForFileCommand ??
                    (_browseForFileCommand = new DelegateCommand(ExecuteBrowseForFileCommand, CanExecuteBrowseForFileCommand));
            }
        }

        public bool CanExecuteBrowseForFileCommand(object parameter)
        {
            return true;
        }

        public async void ExecuteBrowseForFileCommand(object parameter)
        {
            var dlg = new OpenFileDialog();

            if (ContainerType == (int)ContainerTypeEnum.S100_EXCHANGESET)
                dlg.Filter = "S100 EexchangeSet files|*.zip";
            else if (ContainerType == (int)ContainerTypeEnum.S100_DATASET)
                dlg.Filter = "S100 Dataset files|*.xml";
            else
            {
                if(DataProductType == (int)Secom_DataProductTypeEnum.RTZ)
                    dlg.Filter = "RTZ files|*.rtz";
                else
                    dlg.Filter = "Any file|*.*";
            }

            if (dlg.ShowDialog() == true)
            {
                File = dlg.FileName;
            }

        }

        private ICommand _browseForPublicKeyCommand;
        public ICommand BrowseForPublicKeyCommand
        {
            get
            {
                return _browseForPublicKeyCommand ??
                    (_browseForPublicKeyCommand = new DelegateCommand(ExecuteBrowseForPublicKeyCommand, CanExecuteBrowseForPublicKeyCommand));
            }
        }

        public bool CanExecuteBrowseForPublicKeyCommand(object parameter)
        {
            return DataProtection;
        }

        public void ExecuteBrowseForPublicKeyCommand(object parameter)
        {
            if (!DataProtection)
            {
                MessageBox.Show("Receivers public key is only needed if dataexcryption is enabled");
                return;
            }

            var dlg = new OpenFileDialog();
            dlg.Filter = "Certifikat|*.pem;*.crt|All files|*.*";

            if (dlg.ShowDialog() == true)
            {
                PublicKey = dlg.FileName;
            }
        }
    }
}