﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;
using System.Windows;
using System.Windows.Input;
using SECOM.Module.Ui.Models;
using SECOM.Common.Models.Public;
using SECOM.Module.Ui.Utils;

namespace SECOM.Module.Ui.ViewModels
{
    public class SendAckDialogViewModel : ViewModelBase
    {
        public SendAckDialogViewModel()
        {
            AckTypes = new List<ComboBoxData>();
            AckTypes.Add(new ComboBoxData { Text = AckTypeEnum.Opened.ToString(), Value = (int)AckTypeEnum.Opened });
            AckTypes.Add(new ComboBoxData { Text = AckTypeEnum.Error.ToString(), Value = (int)AckTypeEnum.Error });

            NackTypes = new List<ComboBoxData>();
            NackTypes.Add(new ComboBoxData { Text = "XML Schema Validation Error", Value = (int)NackTypeEnum.XMLSchemaValidationError });
            NackTypes.Add(new ComboBoxData { Text = "Unknown Data Type Or Version", Value = (int)NackTypeEnum.UnknownDataProductTypeOrVersion });
            NackTypes.Add(new ComboBoxData { Text = "Failed Signature Verification", Value = (int)NackTypeEnum.FailedSignatureVerification });
            NackTypes.Add(new ComboBoxData { Text = "Failed Decryption", Value = (int)NackTypeEnum.FailedDecryption });
            NackTypes.Add(new ComboBoxData { Text = "FailedDecompression", Value = (int)NackTypeEnum.FailedDecompression });
            NackTypes.Add(new ComboBoxData { Text = "", Value = 999 });

        }

        private string _publicKey = "";
        public string PublicKey
        {
            get
            {
                return _publicKey;
            }
            set
            {
                if (_publicKey == value)
                    return;
                _publicKey = value;

                OnPropertyChanged(() => PublicKey);
            }
        }

        private Guid _transactionId;
        public Guid TransactionId
        {
            get
            {
                return _transactionId;
            }
            set
            {
                if (_transactionId == value)
                    return;
                _transactionId = value;

                OnPropertyChanged(() => TransactionId);
            }
        }

        private int _ackType = 0;
        public int AckType
        {
            get
            {
                return _ackType;
            }
            set
            {
                if (_ackType == value)
                    return;
                _ackType = value;

                OnPropertyChanged(() => AckType);
            }
        }

        private int _nackType = 999;
        public int NackType
        {
            get
            {
                return _nackType;
            }
            set
            {
                if (_nackType == value)
                    return;
                _nackType = value;

                OnPropertyChanged(() => NackType);
            }
        }

        public List<ComboBoxData> AckTypes { get; private set; }
        public List<ComboBoxData> NackTypes { get; private set; }

        private ICommand _browseForPublicKeyCommand;
        public ICommand BrowseForPublicKeyCommand
        {
            get
            {
                return _browseForPublicKeyCommand ??
                    (_browseForPublicKeyCommand = new DelegateCommand(ExecuteBrowseForPublicKeyCommand, CanExecuteBrowseForPublicKeyCommand));
            }
        }

        public bool CanExecuteBrowseForPublicKeyCommand(object parameter)
        {
            return true;
        }

        public void ExecuteBrowseForPublicKeyCommand(object parameter)
        { 
            var dlg = new OpenFileDialog();
            dlg.Filter = "Certifikat|*.pem;*.crt|All files|*.*";

            if (dlg.ShowDialog() == true)
            {
                PublicKey = dlg.FileName;
            }
        }
    }
}
