﻿using Microsoft.Win32;
using SECOM.Module.Ui.Models;
using SECOM.Module.Ui.Services;
using SECOM.Module.Ui.Utils;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.Json;
using System.Windows.Input;

namespace SECOM.Module.Ui.ViewModels
{
    public class SettingsViewModel : ViewModelBase
    {
        private SettingsService _settingsService;

        public SettingsViewModel(SettingsService settingsService)
        {
            _settingsService = settingsService;

            var executableLocation = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            var jsonFile = Path.Combine(executableLocation, "EnvironmentConfig.json");

            Environments = new ObservableCollection<SecomEnvironment>();
            var json = File.ReadAllText(jsonFile);
            JsonSerializer.Deserialize<List<SecomEnvironment>>(json).ForEach(Environments.Add);

            if (Environments != null)
                SelectedEnvironment = Environments.FirstOrDefault(x => x.Name == _settingsService.Environment);

            if (SelectedEnvironment != null)
                SelectedInstance = SelectedEnvironment.Instances.FirstOrDefault(x => x.Name == _settingsService.InstanceName);
        }

        private ObservableCollection<SecomEnvironment> _environments;
        public ObservableCollection<SecomEnvironment> Environments
        {
            get
            {
                return _environments;
            }
            set
            {
                if (_environments == value)
                    return;

                _environments = value;

                OnPropertyChanged(() => Environments);
            }
        }

        private SecomEnvironment _selectedEnvironment;
        public SecomEnvironment SelectedEnvironment
        {
            get
            {
                return _selectedEnvironment;
            }
            set
            {
                if (_selectedEnvironment == value)
                    return;

                _selectedEnvironment = value;

                if (_selectedEnvironment == null)
                {
                    _settingsService.Environment = "Not selected";
                }
                else
                {
                    _settingsService.Environment = _selectedEnvironment.Name;
                }

                OnPropertyChanged(() => SelectedEnvironment);
            }
        }

        private SecomInstance _selectedInstance;
        public SecomInstance SelectedInstance
        {
            get
            {
                return _selectedInstance;
            }
            set
            {
                if (_selectedInstance == value)
                    return;

                _selectedInstance = value;

                if (_selectedInstance == null)
                {
                    _settingsService.InstanceDisplayName = "Not selected";
                    _settingsService.InstanceName = "";
                    _settingsService.PrivateBaseUrl = "";
                    _settingsService.PublicBaseUrl = "";
                    _settingsService.CertificateFile = "";
                    _settingsService.CertificatePassword = "";
                    _settingsService.EnvelopeCertificateFile = "";
                    _settingsService.EnvelopeCertificatePassword = "";
                }
                else
                {
                    _settingsService.InstanceDisplayName = _selectedInstance.DisplayName;
                    _settingsService.InstanceName = _selectedInstance.Name;
                    _settingsService.PrivateBaseUrl = _selectedInstance.PrivateBaseUrl;
                    _settingsService.PublicBaseUrl = _selectedInstance.PublicBaseUrl;
                    _settingsService.CertificateFile = _selectedInstance.CertificateFile;
                    _settingsService.CertificatePassword = _selectedInstance.CertificatePassword;
                    _settingsService.EnvelopeCertificateFile = _selectedInstance.EnvelopeCertificateFile;
                    _settingsService.EnvelopeCertificatePassword = _selectedInstance.EnvelopeCertificatePassword;
                }

                OnPropertyChanged(() => SelectedInstance);
            }
        }
    }
}
