﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using SECOM.Pki.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.IO;
using System.Linq;
using System.Text;

namespace SECOM.Pki.Rest.v1
{
    /// <summary>
    /// 
    /// </summary>
    public class CertificateController : Controller
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="caAlias">caAlias</param>
        /// <returns></returns>
        [HttpGet]
        [Route("/crl/{caAlias}")]
        [Produces("application/x-pem-file", new[] { "application/json" })]
        [ProducesResponseType(StatusCodes.Status200OK, Type = typeof(object))]
        [ProducesResponseType(StatusCodes.Status401Unauthorized, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status403Forbidden, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status404NotFound, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status500InternalServerError, Type = typeof(ResponseObject))]
        public IActionResult GetCrl([FromRoute][Required] string caAlias)
        {
            try
            {
                return Ok(crl);
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, new ResponseObject
                {
                    Message = $"{ex.Message}"
                });
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="caAlias">caAlias</param>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("/ocsp/{caAlias}")]
        [Consumes("application/ocsp-request")]
        [Produces("application/ocsp-response", new[] { "application/json" })]
        [ProducesResponseType(StatusCodes.Status200OK, Type = typeof(byte[]))]
        [ProducesResponseType(StatusCodes.Status201Created, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status401Unauthorized, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status403Forbidden, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status404NotFound, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status500InternalServerError, Type = typeof(ResponseObject))]
        public IActionResult PostOcsp([FromRoute][Required] string caAlias, [FromBody][Required] byte[] input)
        {
            try
            {
                return Ok(new MemoryStream(Encoding.UTF8.GetBytes("OCSP response")));
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, new ResponseObject
                {
                    Message = $"{ex.Message}"
                });
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="caAlias">caAlias</param>
        /// <returns></returns>
        [HttpGet]
        [Route("/ocsp/{caAlias}/**")]
        [Produces("application/ocsp-response", new[] { "application/json" })]
        [ProducesResponseType(StatusCodes.Status200OK, Type = typeof(byte[]))]
        [ProducesResponseType(StatusCodes.Status401Unauthorized, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status403Forbidden, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status404NotFound, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status500InternalServerError, Type = typeof(ResponseObject))]
        public IActionResult GetOcsp([FromRoute][Required] string caAlias)
        {
            try
            {
                return Ok(new MemoryStream(Encoding.UTF8.GetBytes("OCSP response")));
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, new ResponseObject
                {
                    Message = $"{ex.Message}"
                });
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="certId">The serial number of the certificate given in decimal</param>
        /// <returns></returns>
        [HttpGet]
        [Route("/publicKey/{certId}")]
        [Produces("application/x-pem-file", new[] { "application/json" })]
        [ProducesResponseType(StatusCodes.Status200OK, Type = typeof(byte[]))]
        [ProducesResponseType(StatusCodes.Status401Unauthorized, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status403Forbidden, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status404NotFound, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status500InternalServerError, Type = typeof(ResponseObject))]
        public IActionResult GetPublicKey([FromRoute][Required] string certId)
        {
            try
            {
                return Ok(new MemoryStream(Encoding.UTF8.GetBytes(publicKey)));
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, new ResponseObject
                {
                    Message = $"{ex.Message}"
                });
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="csr">A PEM encoded PKCS#10 CSR</param>
        /// <returns></returns>
        [HttpPost]
        [Route("/csr")]
        [Consumes("text/plain")]
        [Produces("application/pem-certificate-chain", new[] { "application/json"})]
        [ProducesResponseType(StatusCodes.Status200OK, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status201Created, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status401Unauthorized, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status403Forbidden, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status404NotFound, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status500InternalServerError, Type = typeof(ResponseObject))]
        public IActionResult SignRequest([FromBody][Required] string csr)
        {
            try
            {
                return Ok(new ResponseObject { Message = "Signed cert" });
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, new ResponseObject
                {
                    Message = $"{ex.Message}"
                });
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="certId">The serial number of the certificate given in decimal</param>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("/revoke/{certId}")]
        [Consumes("application/json")]
        [Produces("application/json")]
        [ProducesResponseType(StatusCodes.Status200OK, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status201Created, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status401Unauthorized, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status403Forbidden, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status404NotFound, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status500InternalServerError, Type = typeof(ResponseObject))]
        public IActionResult Revoke([FromRoute][Required] string certId, [FromBody][Required] CertificateRevocation input)
        {
            try
            {
                return Ok(new ResponseObject { Message = "Revoked cert" });
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, new ResponseObject
                {
                    Message = $"{ex.Message}"
                });
            }
        }

        private const string publicKey = @"-----BEGIN PUBLIC KEY-----
MHYwEAYHKoZIzj0CAQYFK4EEACIDYgAEDzgt+kb+Xc61shuT7DssPakYP3puKIPA
xp8cIOK6cBh24eKztu13RFTNUGKL+UoQl3Ngbt2aM9Qpt8fEDXL8e9sKuIlUdX9D
Lnwf08MxRQmdMsZ/uf1MCMNch9rhpGXy
-----END PUBLIC KEY-----";

        private const string crl = @"-----BEGIN X509 CRL-----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-----END X509 CRL-----";
    }
}