﻿using AutoMapper;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using Org.BouncyCastle.Math.EC.Rfc7748;
using SECOM.Common.Models.Private;
using SECOM.Common.Models.Public;
using SECOM.DataAccess;
using System;
using System.Threading.Tasks;
using System.Collections.Generic;
using System.Text;
using SECOM.Common.Services;
using System.Linq;

namespace SECOM.Service.Private.Rest.v1
{
    [Route("v1/[controller]")]
    [ApiController]
    public class AcknowledgementController : ControllerBase
    {
        private readonly ILogger<AcknowledgementController> _logger;
        private SecomDbContext _context;

        public AcknowledgementController(SecomDbContext context, ILogger<AcknowledgementController> logger)
        {
            _context = context;
            _logger = logger;
        }

        [HttpPost]
        public async Task<ActionResult<Guid>> Post([FromBody] AcknowledgementObject ackObj)
        {

            var updateEntity = _context.UploadedMessage.Where(x => x.TransactionIdentifier == ackObj.Envelope.TransactionIdentifier).FirstOrDefault();
            if(updateEntity == null)
            {
                _logger.LogError($"Failed to find message with id {ackObj.Envelope.TransactionIdentifier}.");
                return BadRequest(new ResponseObject
                {
                    Message = $"Failed to find message with id {ackObj.Envelope.TransactionIdentifier}."
                });
            }

            updateEntity.OperationalAckDelivered = true;
            await _context.SaveChangesAsync();

            return Ok(ackObj.Envelope.TransactionIdentifier);
        }
    }
}
