﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using Org.BouncyCastle.Math.EC.Rfc7748;
using SECOM.Common.Models.Private;
using SECOM.Common.Models.Public;
using SECOM.DataAccess;

namespace SECOM.Service.Private.Rest.v1
{
    [Route("v1/[controller]")]
    [ApiController]
    public class EncryptionKeyController : ControllerBase
    {
        private readonly ILogger _logger;
        private readonly SecomDbContext _dbContext;
        private readonly IMapper _mapper;

        public EncryptionKeyController(ILogger<EncryptionKeyController> logger,
            SecomDbContext dbContext,
            IMapper mapper,
            IHttpContextAccessor contextAccessor)
        {
            _logger = logger;
            _dbContext = dbContext;
            _mapper = mapper;

            _logger.LogDebug($"{GetType().FullName} accessed by user {contextAccessor.HttpContext.User.Identity.Name} from {contextAccessor.HttpContext.Connection.RemoteIpAddress}");

        }

        [HttpGet]
        public async Task<List<EncryptionKeyDto>> GetAll()
        {
            return _mapper.Map<List<EncryptionKeyDto>>(
                await _dbContext.EncryptionKey.Include(x => x.DigitalSignature).ToListAsync());
        }

        [HttpGet]
        [Route("{transactionIdentifier}")]
        public async Task<EncryptionKeyDto> GetOne([FromRoute]Guid transactionIdentifier)
        {
            var dbEntity = await _dbContext.EncryptionKey
                .Include(x => x.DigitalSignature)
                .FirstOrDefaultAsync(x => x.TransactionIdentifier == transactionIdentifier);
            
            if (dbEntity == null)
                return null;

            return _mapper.Map<EncryptionKeyDto>(dbEntity);
        }

        [HttpPost]
        public async Task<Guid> Post([FromBody] EncryptionKeyDto keyObject)
        {
            var dbEntity = _mapper.Map<DataAccess.Entities.EncryptionKey>(keyObject);
            await _dbContext.EncryptionKey.AddAsync(dbEntity);
            await _dbContext.SaveChangesAsync();
            return dbEntity.TransactionIdentifier;
        }

        [HttpDelete]
        [Route("{transactionIdentifier}")]
        public async Task Delete([FromRoute]Guid transactionIdentifier)
        {
            var dbEntity = await _dbContext.EncryptionKey.FirstOrDefaultAsync(x => x.TransactionIdentifier == transactionIdentifier);
            if (dbEntity == null)
                return;

            _dbContext.EncryptionKey.Remove(dbEntity);
            await _dbContext.SaveChangesAsync();
        }
    }
}