﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using SECOM.Common.Models.Private;
using SECOM.Common.Models.Public;
using SECOM.DataAccess;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using AutoMapper;
using Microsoft.AspNetCore.Http;
using SECOM.DataAccess.Entities;

namespace SECOM.Service.Private.Rest.v1
{
    [Route("v1/[controller]")]
    [ApiController]
    public class NotificationController : ControllerBase
    {
        private readonly ILogger _logger;
        private readonly SecomDbContext _dbContext;
        private readonly IMapper _mapper;

        public NotificationController(ILogger<NotificationController> logger,
            SecomDbContext dbContext,
            IMapper mapper,
            IHttpContextAccessor contextAccessor)
        {
            _logger = logger;
            _dbContext = dbContext;
            _mapper = mapper;

            _logger.LogDebug($"{GetType().FullName} accessed by user {contextAccessor.HttpContext.User.Identity.Name} from {contextAccessor.HttpContext.Connection.RemoteIpAddress}");
        }

        [HttpGet]
        public async Task<List<NotificationDto>> GetNotification()
        {
            var notifications = await _dbContext.Notification.Where(x => x.Notified == false).OrderByDescending(x => x.ReceivedAt).Take(20).ToListAsync();

            foreach(var notification in notifications)
            {
                notification.Notified = true;
                notification.NotifiedAt = DateTime.UtcNow;
            }
            await _dbContext.SaveChangesAsync();

            return _mapper.Map<List<NotificationDto>>(notifications);
        }
    }
}
