﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using SECOM.Common.Models.Public;
using SECOM.DataAccess;
using SECOM.DataAccess.Entities;

namespace SECOM.Service.Private.Rest.v1
{
    [Route("v1/[controller]")]
    [ApiController]
    public class SentMessageController : ControllerBase
    {
        private SecomDbContext _context;
        private readonly IHttpContextAccessor _contextAccessor;

        public SentMessageController(SecomDbContext context, IHttpContextAccessor contextAccessor)
        {
            _context = context;
            _contextAccessor = contextAccessor;
        }

        [HttpPost]
        public async Task<bool> Post([FromBody] UploadObject uploadObj, string receiverMrn)
        {
            try
            {
                SentMessage dbEntity = ConvertToEntity(uploadObj);
                dbEntity.ReceiverMrn = receiverMrn;

                await _context.SentMessage.AddAsync(dbEntity);
                await _context.SaveChangesAsync();

                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

        [Route("link")]
        [HttpPost]
        public async Task<bool> PostLink([FromBody] UploadObject uploadObj, string receiverMrn, DateTime timeToLive)
        {
            try
            {
                if (_contextAccessor.HttpContext.Request.Host.Host.ToLower().Contains("localhost"))
                {
                    //receiverMrn = $@"urn:mrn:mcp:user:navelink-dev:sma:extperflo01";
                    receiverMrn = $@"urn:mrn:mcp:service:navelink-dev:sma:instance:secom:secomtest01";
                }

                SentMessage dbEntity = ConvertToEntity(uploadObj);
                dbEntity.ReceiverMrn = receiverMrn;
                dbEntity.TimeToLive = timeToLive;
                dbEntity.SentAsLink = true;
                await _context.SentMessage.AddAsync(dbEntity);
                await _context.SaveChangesAsync();

                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }


        private SentMessage ConvertToEntity(UploadObject uploadObj)
        {
            return new SentMessage
            {
                AckReceived = false,
                Data = uploadObj.Envelope.Data,
                ContainerType = (int)uploadObj.Envelope.ContainerType,
                ExchangeMetadata = new DataAccess.Entities.ExchangeMetadata
                {
                    CompressionFlag = uploadObj.Envelope.ExchangeMetadata.CompressionFlag,
                    DataProtection = uploadObj.Envelope.ExchangeMetadata.DataProtection,
                    DigitalSignature = new DigitalSignature
                    {
                        DigitalSignatureValue = uploadObj.Envelope.ExchangeMetadata.DigitalSignatureValue.DigitalSignature,
                        PublicCertificate = uploadObj.Envelope.ExchangeMetadata.DigitalSignatureValue.PublicCertificate,
                        PublicRootCertificateThumbprint = uploadObj.Envelope.ExchangeMetadata.DigitalSignatureValue.PublicRootCertificateThumbprint
                    },
                    DigitalSignatureReference = uploadObj.Envelope.ExchangeMetadata.DigitalSignatureReference,
                    ProtectionScheme = uploadObj.Envelope.ExchangeMetadata.ProtectionScheme
                },
                SentTime = DateTime.UtcNow,
                ToSubscription = uploadObj.Envelope.FromSubscription,
                TransactionIdentifier = uploadObj.Envelope.TransactionIdentifier,
            };
        }
    }
}