﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Configuration;
using SECOM.Common.Integration.Models;
using System.Security.Cryptography.X509Certificates;
using Microsoft.AspNetCore.Http;
using IConfiguration = Microsoft.Extensions.Configuration.IConfiguration;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;

namespace SECOM.Service.Public.BackgroundTask
{
    public interface ICallService
    {
        Task<CallServiceResponseDto> DoCall(CallServiceRequestDto callServiceObj);
    }
    public class CallService : ICallService
    {
        private readonly IConfiguration _configuration;
        private readonly ILogger _logger;
        public CallService(IConfiguration configuration, ILogger<CallService> logger)
        {
            _configuration = configuration;
            _logger = logger;
        }
        public async Task<CallServiceResponseDto> DoCall(CallServiceRequestDto callServiceObj)
        {
            var result = new CallServiceResponseDto();

            var path = _configuration.GetValue<string>("ASPNETCORE_Kestrel:Certificates:Default:Path");
            var password = _configuration.GetValue<string>("ASPNETCORE_Kestrel:Certificates:Default:Password");
            _logger.LogDebug($"Cert path: {path}");

            var sslCert = new X509Certificate2(path, password);
            var clientHandler = new HttpClientHandler();
            clientHandler.ClientCertificates.Add(sslCert);
            clientHandler.ClientCertificateOptions = ClientCertificateOption.Manual;
            clientHandler.ServerCertificateCustomValidationCallback = (message, cert, chain, errors) =>
            {
                return true;
            };

            using (var client = new HttpClient(clientHandler))
            {
                client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
                client.DefaultRequestHeaders.CacheControl = new CacheControlHeaderValue
                {
                    NoCache = true,
                    NoStore = true,
                    MaxAge = new TimeSpan(0),
                    MustRevalidate = true
                };
                client.DefaultRequestHeaders.ConnectionClose = true;
                if (callServiceObj.Body != null)
                {
                    var uploadContent = new StringContent(callServiceObj.Body, Encoding.UTF8, "application/json");
                    using (HttpResponseMessage response = await client.PostAsync(callServiceObj.EndpointMethod, uploadContent))
                    {
                        result.StatusCode = response.StatusCode;
                        result.Body = await response.Content.ReadAsStringAsync();
                    }
                }
                else
                {
                    using (HttpResponseMessage response = await client.GetAsync(callServiceObj.EndpointMethod))
                    {
                        result.StatusCode = response.StatusCode;
                        result.Body = await response.Content.ReadAsStringAsync();
                    }
                }
            }
            _logger.LogInformation($"Returning result!");
            return result;
        }
    }


}
