﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Http;
using System.Security.Authentication;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.Http;
using SECOM.Common.Services;

namespace SECOM.Service.Public
{
    public class AuthenticationActionFilter : IActionFilter
    {
        private readonly ILogger _logger;

        public AuthenticationActionFilter(ILogger logger)
        {
            _logger = logger;
        }
        public void OnActionExecuted(ActionExecutedContext context)
        {
            _logger.LogInformation($"Incoming request from {context.HttpContext.Connection.RemoteIpAddress} to {GetUri(context.HttpContext.Request)}.");
            var serviceId = string.Empty;
            var orgId = string.Empty;

            try
            {
                if (context.ActionDescriptor.DisplayName.ToLower() == "ping")
                {
                    serviceId = "Ping";
                    orgId = "Ping";
                }
                else
                {
                    string certString = string.Empty;
                    var cert = context.HttpContext.Connection.ClientCertificate;
                    if (cert == null)
                    {
                        certString = context.HttpContext.Request.Headers["X-ARR-ClientCert"];
                    }

                   // bool valid = new IdentityRegistryService(_logger).IsCertificateValid(cert);
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex.Message, ex);
                throw;
            }

            _logger.LogInformation($"Authenticated call from service id: {serviceId}, org: {orgId} to url: {GetUri(context.HttpContext.Request)}.");
        }

        public void OnActionExecuting(ActionExecutingContext context)
        {
            throw new NotImplementedException();
        }

        private static Uri GetUri(HttpRequest request)
        {
            var builder = new UriBuilder();
            builder.Scheme = request.Scheme;
            builder.Host = request.Host.Value;
            builder.Path = request.Path;
            builder.Query = request.QueryString.ToUriComponent();
            return builder.Uri;
        }
    }

}
