﻿using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Http;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;

namespace SECOM.Service.Public.Filters
{
    public class ExceptionMiddleware
    {
        private readonly RequestDelegate _next;

        public ExceptionMiddleware(RequestDelegate next)
        {
            _next = next;
        }

        public async Task Invoke(HttpContext context)
        {
            try
            {
                // Call the next delegate/middleware in the pipeline
                await _next(context);
            }
            catch (Exception ex)
            {
                await HandleExceptionAsync(context, ex);
            }
        }

        private static async Task HandleExceptionAsync(HttpContext context, Exception exception)
        {
            if (exception is Microsoft.AspNetCore.Server.IIS.BadHttpRequestException badRequestExceptionIIS && badRequestExceptionIIS.Message == "Request entity too large.")
            {
                context.Response.StatusCode = (int)HttpStatusCode.RequestEntityTooLarge;
            }
            if (exception is Microsoft.AspNetCore.Server.Kestrel.Core.BadHttpRequestException badRequestExceptionKestrel && badRequestExceptionKestrel.Message == "Request entity too large.")
            {
                context.Response.StatusCode = (int)HttpStatusCode.RequestEntityTooLarge;
            }
        }
    }
}
