﻿using System;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using SECOM.Common.Models.Public;

namespace SECOM.Service.Public.Rest.v1
{
    [Route("v1/ping")]
    [ApiController]
    public class PingController : ControllerBase
    {
        /// <summary>
        /// The purpose of the interface is to provide a dynamic method to ask for the technical status of the specific service instance.
        /// </summary>
        /// <returns>Status of the service reported at runtime by the service instance</returns>
        /// <response code="200">PingResponseObject</response>
        /// <response code="401">Unauthorized</response>
        /// <response code="405">Method not allowed</response>
        /// <response code="500">Internal server error</response>
        [HttpGet]
        [Consumes("application/json")]
        [Produces("application/json")]
        [ProducesResponseType(StatusCodes.Status200OK, Type = typeof(string))]
        [ProducesResponseType(StatusCodes.Status401Unauthorized, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status405MethodNotAllowed, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status500InternalServerError, Type = typeof(ResponseObject))]
        public async Task<ActionResult<PingResponseObject>> Get()
        {
            try
            {
                return Ok(new PingResponseObject
                {
                    LastPrivateInteractionTime = DateTime.Now
                });
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, new ResponseObject
                {
                    Message = $"{ex.Message}"
                });
            }
        }
    }
}
