﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using SECOM.Common.Models.Public;
using SECOM.Common.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using System.Security.Cryptography.X509Certificates;
using IConfiguration = Microsoft.Extensions.Configuration.IConfiguration;
using Microsoft.Extensions.Configuration;

namespace SECOM.Service.Public.Rest.v1
{
    [Route("api/[controller]")]
    [ApiController]
    public class PublicKeyController : ControllerBase
    {
        private readonly ILogger _logger;
        private readonly ICertificateService _certificateService;
        private readonly IConfiguration _configuration;

        public PublicKeyController(ILogger<PublicKeyController> logger,
            ICertificateService certificateService,
            IConfiguration configuration)
        {
            _logger = logger;
            _certificateService = certificateService;
            _configuration = configuration;
        }

        /// <summary>
        /// The purpose of the interface is to request a public key 
        /// </summary>
        /// <param name="dataProtection ">Flag indicating public certificate needed for encryption key exchange</param>
        /// <param name="certificateThumbprint">Claimed Thumbprint for Signed Key (X.509 Certificate)</param>
        /// <returns>A public certificate or an error message</returns>
        /// <response code="200">PublicKeyResponseObject</response>
        /// <response code="400">Bad request</response>
        /// <response code="401">Unauthorized</response>
        /// <response code="403">Not authorized to requested information</response>
        /// <response code="404">Public key not found</response>
        /// <response code="405">Method not allowed</response>
        /// <response code="500">Internal server error</response>
        /// <response code="501">Not implemented</response>
        [HttpGet]
        [Consumes("application/json")]
        [Produces("application/json")]
        [ProducesResponseType(StatusCodes.Status200OK, Type = typeof(PublicKeyResponseObject))]
        [ProducesResponseType(StatusCodes.Status400BadRequest, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status401Unauthorized, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status403Forbidden, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status404NotFound, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status405MethodNotAllowed, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status500InternalServerError, Type = typeof(ResponseObject))]
        [ProducesResponseType(StatusCodes.Status501NotImplemented, Type = typeof(ResponseObject))]
        public async Task<ActionResult<PublicKeyResponseObject>> Get(
            bool? dataProtection,
            string certificateThumbprint)
        {
            var path = _configuration.GetValue<string>("ASPNETCORE_Kestrel:Certificates:Default:Path");
            var password = _configuration.GetValue<string>("ASPNETCORE_Kestrel:Certificates:Default:Password");
            var sslCert = new X509Certificate2(path, password, X509KeyStorageFlags.Exportable);

            return Ok(new PublicKeyResponseObject
            {
                PublicCertificate = _certificateService.GetBase64SingleLine(sslCert)
            });
        }
    }
}
