﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.Serialization;
using System.Threading.Tasks;

namespace SECOM.ServiceDiscovery.Models
{
    [DataContract]
    public class ServiceInstance
    {
        /// <summary>
        /// Gets or sets InstanceId MRN Service instance identity (refer IALA G1143) 
        /// </summary>
        [DataMember(Name = "instanceId")]
        public string InstanceId { get; set; }

        /// <summary>
        /// Gets or sets version S100_OC_ServiceMetaData, version of the service instance 
        /// </summary>
        [DataMember(Name = "version")]
        public string Version { get; set; }

        /// <summary>
        /// Gets or sets Name S100_OC_ServiceMetaData, name on the service instance
        /// </summary>
        [DataMember(Name ="name")]
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets Status S100_OC_StatusType, status on the service
        /// </summary>
        [DataMember(Name ="status")]
        public string Status { get; set; }

        /// <summary>
        /// Gets or sets Description S100_OC_ServiceMetaData, description of the service instance
        /// </summary>
        [DataMember(Name ="description")]
        public string Description { get; set; }

        /// <summary>
        /// Gets or sets OrganizationId MRN, organization identity of the registrator  (refer IALA G1143) 
        /// </summary>
        [DataMember(Name = "organizationId")]
        public string OrganizationId { get; set; }

        /// <summary>
        /// Gets or sets EndpointUri URI, endpoint to the service instance 
        /// </summary>
        [DataMember(Name = "endpointUri")]
        public Uri EndpointUri { get; set; }

        /// <summary>
        /// Gets or sets EndpointType S100_OC_ServiceTechnology i.e. REST
        /// </summary>
        [DataMember(Name = "endpointType")]
        [DefaultValue("REST")]
        public string EndpointType { get; set; }

        /// <summary>
        /// Gets or sets Keywords S100_OC_ServiceMetaData, a list of keywords associated with the service Searchable keywords for discoverability. 
        /// </summary>
        [DataMember(Name ="keywords")]
        public string Keywords { get; set; }

        /// <summary>
        /// Gets or sets Unlocode Geolocation by UnLoCode  (refer UN/LOCODE Code List 2020-2) 
        /// </summary>
        [DataMember(Name ="unlocode")]
        public string Unlocode { get; set; }

        /// <summary>
        /// Gets or sets InstanceAsXml Original XML based on service registry used. (refer IALA G1128) 
        /// </summary>
        [DataMember(Name = "instanceAsXml")]
        public string InstanceAsXml { get; set; }
    }
}
