﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace SECOM.ServiceDiscovery.Services
{
    public class FormatValidator
    {
        private const string UVID_FORMAT = @"^urn:mrn:stm:voyage:id:[\w-]+:[\w-]+$";
        private const string ORG_MRN_FORMAT = @"^urn:mrn:stm:org:[\w-]+$";
        private const string SERVICE_MRN_FORMAT = @"^urn:mrn:stm:service:instance:[\w-]+:[\w-]+$";
        private const string SERVICE_FREETEXT = @"^[a-zA-Z0-9 +_:\-,.]*$";
        private const string UNLOCODE_FORMAT = @"^[a-zA-Z]{2}[a-zA-Z2-9]{3}";
        private const string MMSI_FORMAT = @"^(MID\d{6}|0MID\d{5}|00MID\{4})";
        private const string IMO_FORMAT = @"^\d{7}(?:\d{2})?$";

        public static bool IsValidOrgId(string orgId)
        {
            Regex regex = new Regex(ORG_MRN_FORMAT);
            return regex.IsMatch(orgId);
        }

        public static bool IsValidServiceId(string serviceId)
        {
            Regex regex = new Regex(SERVICE_MRN_FORMAT);
            return regex.IsMatch(serviceId);
        }

        public static bool IsValidUvid(string uvid)
        {
            Regex regex = new Regex(UVID_FORMAT);
            return regex.IsMatch(uvid);
        }

        public static bool IsValidFreeText(string freeText)
        {
            Regex regex = new Regex(SERVICE_FREETEXT);
            return regex.IsMatch(freeText);
        }

        public static bool IsValidUnlocode(string unlocode)
        {
            Regex regex = new Regex(UNLOCODE_FORMAT);
            return regex.IsMatch(unlocode);
        }

        public static bool IsValidMmsi(string mmsi)
        {
            Regex regex = new Regex(MMSI_FORMAT);
            return regex.IsMatch(mmsi);
        }

        public static bool IsValidImo(string imo)
        {
            Regex regex = new Regex(IMO_FORMAT);
            return regex.IsMatch(imo);
        }
    }
}
