﻿using SECOM.ServiceDiscovery.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace SECOM.ServiceDiscovery.Services
{
    public interface IServiceRegistryService
    {
        Task<SearchServiceResponse> FindService(string url, bool includeXml = false);
    }

    public class ServiceRegistryService : IServiceRegistryService
    {
        public readonly HttpClient _client;
        private bool _includeXml;

        public ServiceRegistryService(IHttpClientFactory httpClientFactory)
        {
            _client = httpClientFactory.CreateClient("ServiceRegistry");
        }
        public async Task<SearchServiceResponse> FindService(string url, bool includeXml = false)
        {
            try
            {
                _includeXml = includeXml;

                var response = await _client.GetAsync(url);
                response.EnsureSuccessStatusCode();

                using var responseStream = await response.Content.ReadAsStreamAsync();
                {
                    var serializeOptions = new JsonSerializerOptions
                    {
                        PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
                    };

                    var result =  await JsonSerializer.DeserializeAsync<List<FindServiceResponseObject>>
                        (responseStream, serializeOptions);

                    return ConvertToContract(result);
                }
            }
            catch (Exception ex)
            {
                throw;
            }
        }

        private SearchServiceResponse ConvertToContract(List<FindServiceResponseObject> dataCollection)
        {
            var result = new SearchServiceResponse();
            result.ServiceInstances = new List<ServiceInstance>();
            
            foreach (var data in dataCollection)
            {
                ServiceInstance resultObj = ConvertData(data);
                result.ServiceInstances.Add(resultObj);
            }

            return result;
        }

        private ServiceInstance ConvertData(FindServiceResponseObject data)
        {
            var result = new ServiceInstance
            {
                Description = data.Comment,
                EndpointUri = new Uri(data.EndpointUri, UriKind.Absolute),
                EndpointType = string.IsNullOrEmpty(data.EndpointType) ? "REST" : data.EndpointType,
                InstanceId = data.InstanceId,
                OrganizationId = data.OrganizationId,
                Keywords = data.Keywords,
                Name = data.Name,
                Status = data.Status,
                Unlocode = data.Unlocode,
                Version = data.Version
            };

            if (_includeXml)
                result.InstanceAsXml = data.InstanceAsXml.Content;            

            return result;
        }
    }
}
